@@ -26,6 +26,7 @@
 #include "utils/tools.h"
 #include "tree/mexttree.h"
 #include "phylotesting.h"
+#include "treetesting.h"
 #include "tree/upperbounds.h" // Olga: functions for Upper Bounds analysis
 #include "utils/pllnni.h"
 
@@ -38,6 +39,11 @@ class IQTree;
 */
 void runPhyloAnalysis(Params &params, Checkpoint *checkpoint);
 
+/**
+    Perform separate tree inference across partitions
+ */
+void runUnlinkedPhyloAnalysis(Params &params, Checkpoint *checkpoint);
+
 void startTreeReconstruction(Params &params, IQTree* &iqtree,
         ModelCheckpoint &model_info);
 
