@@ -212,7 +212,8 @@ void ModelDNA::saveCheckpoint() {
     // up to model_parameters[num_params]
 //    setVariables(model_parameters);
     startCheckpoint();
-    CKP_ARRAY_SAVE(6, rates);
+    if (!fixed_parameters)
+        CKP_ARRAY_SAVE(6, rates);
     endCheckpoint();
     ModelMarkov::saveCheckpoint();
 }
@@ -221,7 +222,8 @@ void ModelDNA::restoreCheckpoint() {
   // curiously, this seems to be the only plase ModelDNA uses model_parameters.
     ModelMarkov::restoreCheckpoint();
     startCheckpoint();
-    CKP_ARRAY_RESTORE(6, rates);
+    if (!fixed_parameters)
+        CKP_ARRAY_RESTORE(6, rates);
     endCheckpoint();
 //    getVariables(model_parameters);       // updates rates and state_freq
     string rate_spec = param_spec;
@@ -290,17 +292,20 @@ void ModelDNA::readRates(string str) throw(const char*) {
 string ModelDNA::getNameParams() {
 	if (num_params == 0) return name;
 	ostringstream retname;
-	retname << name << '{';
-	int nrates = getNumRateEntries();
-	int k = 0;
-	for (int i = 0; i < nrates; i++) {
-		if (param_spec[i] > k) {
-			if (k>0) retname << ',';
-			retname << rates[i];
-			k++;
-		}
-	}
-	retname << '}';
+    retname << name;
+    if (!fixed_parameters) {
+        retname << '{';
+        int nrates = getNumRateEntries();
+        int k = 0;
+        for (int i = 0; i < nrates; i++) {
+            if (param_spec[i] > k) {
+                if (k>0) retname << ',';
+                retname << rates[i];
+                k++;
+            }
+        }
+        retname << '}';
+    }
     getNameParamsFreq(retname);
 	return retname.str();
 }
@@ -377,8 +382,14 @@ bool ModelDNA::setRateType(string rate_str) {
 			cout << rates[i] << " ";
 		cout << endl;
 	}
-	param_fixed.resize(num_params+1, false);
-	param_fixed[0] = true; // fix the last entry
+    if (param_fixed.size() == num_params + 1) {
+        num_params = 0;
+        for (auto p : param_fixed)
+            if (!p) num_params++;
+    } else {
+        param_fixed.resize(num_params+1, false);
+        param_fixed[0] = true; // fix the last entry
+    }
 	delete [] num_rates;
 	delete [] avg_rates;
 	return true;
@@ -386,10 +397,15 @@ bool ModelDNA::setRateType(string rate_str) {
 
 
 int ModelDNA::getNDim() {
+    if (fixed_parameters)
+        return 0;
 	ASSERT(freq_type != FREQ_UNKNOWN);
 	// possible TO-DO: cache nFreqParams(freq_type) to avoid repeat calls.
 //        return (num_params+nFreqParams(freq_type));
 
+//    if (linked_model && linked_model != this)
+//        return 0;
+    
 	int ndim = num_params;
 	if (freq_type == FREQ_ESTIMATE) 
 		ndim += num_states-1;
@@ -438,10 +454,10 @@ bool ModelDNA::getVariables(double *variables) {
         // 2015-09-07: relax the sum of state_freq to be 1, this will be done at the end of optimization
 		int ndim = getNDim();
 		changed |= memcmpcpy(state_freq, variables+(ndim-num_states+2), (num_states-1)*sizeof(double));
-                double sum = 0;
-                for (i = 0; i < num_states-1; i++) 
-                        sum += state_freq[i];
-                state_freq[num_states-1] = 1.0 - sum;
+//                double sum = 0;
+//                for (i = 0; i < num_states-1; i++)
+//                        sum += state_freq[i];
+//                state_freq[num_states-1] = 1.0 - sum;
     } else {
         // BQM: for special DNA freq stuffs from MDW
         changed |= freqsFromParams(state_freq,variables+num_params+1,freq_type);
