@@ -20,7 +20,7 @@
 #ifndef MODELDNA_H
 #define MODELDNA_H
 
-#include "modelgtr.h"
+#include "modelmarkov.h"
 #include <string>
 
 /**
@@ -38,14 +38,15 @@ All DNA models are managed here
 
 	@author BUI Quang Minh <minh.bui@univie.ac.at>
 */
-class ModelDNA : public ModelGTR
+class ModelDNA : public ModelMarkov
 {
+    friend class ModelPoMo;
 public:
 	/**
 		constructor
 		@param tree associated tree for the model
 	*/
-    ModelDNA(PhyloTree *tree, bool count_rates = true);
+    ModelDNA(PhyloTree *tree);
 
 	/**
 		constructor
@@ -53,14 +54,19 @@ public:
 		@param freq state frequency type
 		@param tree associated phylogenetic tree
 	*/
-    ModelDNA(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree, bool count_rates = true);
+    ModelDNA(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree);
 
 	/**
 		initialization, called automatically by the constructor, no need to call it
 		@param model_name model name, e.g., JC, HKY.
 		@param freq state frequency type
 	*/
-	virtual void init(const char *model_name, string model_params, StateFreqType freq, string freq_params);
+    virtual void init(const char *model_name, string model_params, StateFreqType freq, string freq_params);
+
+    /**
+        start structure for checkpointing
+    */
+    virtual void startCheckpoint();
 
     /**
         save object into the checkpoint
@@ -89,7 +95,7 @@ public:
 		@param rate_spec a string of six letters describing how rates are related
 		@return TRUE if successful, FALSE otherwise
 	*/
-	bool setRateType(const char *rate_spec);
+	bool setRateType(string rate_spec);
 
 	/**
 		return the number of dimensions
