@@ -5,10 +5,8 @@
 #include <assert.h>
 #include <string.h>
 
-#ifdef USE_EIGEN3
 #include <Eigen/Dense>
 #include <unsupported/Eigen/MatrixFunctions>
-#endif
 
 ModelPoMo::ModelPoMo(PhyloTree *tree) : ModelMarkov(tree) {
 }
@@ -31,8 +29,7 @@ ModelPoMo::ModelPoMo(const char *model_name,
 void ModelPoMo::init_mutation_model(const char *model_name,
                                         string model_params,
                                         StateFreqType freq_type,
-                                        string freq_params,
-                                        string pomo_heterozygosity)
+                                        string freq_params)
 {
     // Trick ModelDNA constructor by setting the number of states to 4 (DNA).
     phylo_tree->aln->num_states = n_alleles;
@@ -183,8 +180,7 @@ void ModelPoMo::init(const char *model_name,
     init_mutation_model(model_name,
                         model_params,
                         freq_type,
-                        freq_params,
-                        pomo_heterozygosity);
+                        freq_params);
     init_sampling_method();
     init_boundary_frequencies();
     // Initialize heterozygosity and the scale factor of the mutation rates
@@ -194,7 +190,12 @@ void ModelPoMo::init(const char *model_name,
     init_fixed_parameters(model_params, pomo_heterozygosity);
     set_heterozygosity_boundaries();
     setInitialMutCoeff();
-    rate_matrix = new double[num_states*num_states];
+    //rate_matrix = new double[num_states*num_states];
+    ignore_state_freq = true;
+    normalize_matrix = false;
+    half_matrix = false;
+    delete [] rates;
+    rates = new double[num_states*num_states];
     updatePoMoStatesAndRateMatrix();
     decomposeRateMatrix();
 
@@ -227,7 +228,7 @@ ModelPoMo::~ModelPoMo() {
 double ModelPoMo::computeSumFreqBoundaryStates() {
     int i;
     double norm_boundary = 0.0;
-    for (i = 0; i < 4; i++)
+    for (i = 0; i < n_alleles; i++)
         norm_boundary += freq_boundary_states[i];
     // Should be 1.0!
     if ((norm_boundary > 1.0 + eps) || (norm_boundary < 1.0 - eps))
@@ -260,7 +261,7 @@ void ModelPoMo::setInitialMutCoeff() {
 double ModelPoMo::computeSumFreqPolyStatesNoMut() {
     double norm_polymorphic = 0.0;
     int i, j;
-    for (i = 0; i < 4; i++) {
+    for (i = 0; i < n_alleles; i++) {
         for (j = 0; j < i; j++)
             norm_polymorphic +=
                 2 * freq_boundary_states[i] * freq_boundary_states[j];
@@ -331,11 +332,13 @@ void ModelPoMo::updatePoMoStatesAndRateMatrix () {
         for (j = 0; j < num_states; j++)
             if (i != j) {
                 row_sum +=
-                    (rate_matrix[i*num_states+j] =
+                    (rates[i*num_states+j] =
                      computeProbBoundaryMutation(i, j));
             }
         tot_sum += state_freq[i]*row_sum;
-        rate_matrix[i*num_states+i] = -(row_sum);
+        // diagonal will be handled later, should not assign now
+        //rates[i*num_states+i] = -(row_sum);
+        rates[i*num_states+i] = 0.0;
     }
     // Thu Aug 17 16:11:19 BST 2017; Dom. Normalization is preferred. Then,
     // branch lengths can be interpreted in an easy way (the length equals the
@@ -346,7 +349,7 @@ void ModelPoMo::updatePoMoStatesAndRateMatrix () {
     // Normalize rate matrix such that one event happens per unit time.
     for (int i = 0; i < num_states; i++) {
         for (int j = 0; j < num_states; j++) {
-            rate_matrix[i*num_states+j] /= tot_sum;
+            rates[i*num_states+j] /= tot_sum;
         }
     }
 
@@ -534,7 +537,31 @@ void ModelPoMo::normalizeMutationRates() {
     }
 
     for (int i = 0; i < n_alleles; i++)
-        for (int j = 0; j < n_alleles; j++) m[i*n_alleles+j] *= m_norm;
+      for (int j = 0; j < n_alleles; j++) {
+        m[i*n_alleles+j] *= m_norm;
+        // DEBUG.
+        // cout << setprecision(15);
+        // cout << m[i*n_alleles+j] << endl;
+      }
+
+    // DEBUG.
+    if (verbose_mode >= VB_MED) {
+      cout << "theta_bm before normalization is " << theta_bm << endl;
+      cout << "heterozygosity is " << heterozygosity << endl;
+      for (int i = 0; i < n_alleles; i++) {
+        for (int j = i+1; j < n_alleles; j++) {
+          // The mutation rate matrix entry is the exchangeability times the
+          // target allele frequency.
+          double ex = m[i*n_alleles+j] / freq_boundary_states[j];
+          cout << setprecision(15);
+          cout << "Exchangeability " << i << " to " << j << " is " << ex << endl;
+        }
+      }
+      computeStateFreq();
+      double normc = computeNormConst();
+      theta_bm = (1.0 - normc) / harmonic(N-1);
+      cout << "theta_bm after normalization is " << theta_bm << endl;
+    }
 }
 
 void ModelPoMo::setScale(double new_scale) {
@@ -586,10 +613,11 @@ void ModelPoMo::writeInfo(ostream &out) {
   report(out);
 }
 
+// TODO: s_freqs is not used.
 void ModelPoMo::computeRateMatrix(double **r_matrix, double *s_freqs, int n_states) {
     for (int i = 0; i < n_states; i++) {
         for (int j = 0; j < n_states; j++) {
-            r_matrix[i][j] = rate_matrix[i*n_states+j];
+            r_matrix[i][j] = rates[i*n_states+j];
         }
     }
 }
@@ -786,6 +814,18 @@ void ModelPoMo::report_model_params(ostream &out, bool reset_scale) {
   else
     out << "The reported rates are scaled by a factor of " << scale << "." << endl;;
 
+  // TODO: If verbose, output rate matrix.
+  if (verbose_mode >= VB_MED) {
+    out << "Rate matrix: " << endl;
+    for (int i = 0; i < num_states; i++) {
+      for (int j = 0; j < num_states; j++) {
+        out << setprecision(8);
+        out << setw(8) << rates[i*num_states+j] << " ";
+      }
+      out << endl;
+    }
+  }
+
   // Report rates.
   // Mutation rates.
   double *rs = NULL;
@@ -987,47 +1027,53 @@ void ModelPoMo::restoreCheckpoint() {
 }
 
 // Declaration of helper function; needed by decomposeRateMatrix().
-int computeStateFreqFromQMatrix (double Q[], double pi[], int n, double space[]);
+//int computeStateFreqFromQMatrix (double Q[], double pi[], int n, double space[]);
+
+// void ModelPoMo::decomposeRateMatrix() {
+//     updatePoMoStatesAndRateMatrix();
+//     // Non-reversible.
+//     if (!is_reversible) {
+//         if (phylo_tree->params->matrix_exp_technique == MET_EIGEN_DECOMPOSITION) {
+//             eigensystem_nonrev(rate_matrix, state_freq, eigenvalues, eigenvalues_imag, eigenvectors, inv_eigenvectors, num_states);
+//             return;
+//         }
+//         else if (phylo_tree->params->matrix_exp_technique == MET_SCALING_SQUARING) {
+//             return;
+//         }
+//         else if (phylo_tree->params->matrix_exp_technique == MET_EIGEN3LIB_DECOMPOSITION) {
+//             // Not (yet?) implemented.
+//             // decomposeRateMatrixEigen3lib();
+//             outError("MET_EIGEN3LIB_DECOMPOSITION does not work with PoMo.");
+//         }
+//         else if (phylo_tree->params->matrix_exp_technique == MET_LIE_MARKOV_DECOMPOSITION)
+//             // Not possible?
+//             // decomposeRateMatrixClosedForm();
+//             outError("Matrix decomposition in closed form not available for PoMo.");
+//         else
+//             outError("Matrix decomposition method unknown.");
+//     }
+//     // Reversible.  Alogrithms for symmetric matrizes can be used.
+//     else {
+//         // TODO DS: This leaves room for speed improvements.
+//         // EigenDecomposition::eigensystem_sym() expects a matrix[][]
+//         // object with two indices.  However, it is not used, because
+//         // ModelPoMo::computeRateMatrix() is called anyways from
+//         // within eigensystem_sym().
+// 		double **temp_matrix = new double*[num_states];
+// 		for (int i = 0; i < num_states; i++)
+// 			temp_matrix[i] = new double[num_states];
+// 		eigensystem_sym(temp_matrix, state_freq, eigenvalues, eigenvectors, inv_eigenvectors, num_states);
+// 		for (int i = num_states-1; i >= 0; i--)
+// 			delete [] temp_matrix[i];
+// 		delete [] temp_matrix;
+//     return;
+//     }
+// }
 
 void ModelPoMo::decomposeRateMatrix() {
     updatePoMoStatesAndRateMatrix();
-    // Non-reversible.
-    if (!is_reversible) {
-        if (phylo_tree->params->matrix_exp_technique == MET_EIGEN_DECOMPOSITION) {
-            eigensystem_nonrev(rate_matrix, state_freq, eigenvalues, eigenvalues_imag, eigenvectors, inv_eigenvectors, num_states);
-            return;
-        }
-        else if (phylo_tree->params->matrix_exp_technique == MET_SCALING_SQUARING) {
-            return;
-        }
-        else if (phylo_tree->params->matrix_exp_technique == MET_EIGEN3LIB_DECOMPOSITION) {
-            // Not (yet?) implemented.
-            // decomposeRateMatrixEigen3lib();
-            outError("MET_EIGEN3LIB_DECOMPOSITION does not work with PoMo.");
-        }
-        else if (phylo_tree->params->matrix_exp_technique == MET_LIE_MARKOV_DECOMPOSITION)
-            // Not possible?
-            // decomposeRateMatrixClosedForm();
-            outError("Matrix decomposition in closed form not available for PoMo.");
-        else
-            outError("Matrix decomposition method unknown.");
-    }
-    // Reversible.  Alogrithms for symmetric matrizes can be used.
-    else {
-        // TODO DS: This leaves room for speed improvements.
-        // EigenDecomposition::eigensystem_sym() expects a matrix[][]
-        // object with two indices.  However, it is not used, because
-        // ModelPoMo::computeRateMatrix() is called anyways from
-        // within eigensystem_sym().
-		double **temp_matrix = new double*[num_states];
-		for (int i = 0; i < num_states; i++)
-			temp_matrix[i] = new double[num_states];
-		eigensystem_sym(temp_matrix, state_freq, eigenvalues, eigenvectors, inv_eigenvectors, num_states);
-		for (int i = num_states-1; i >= 0; i--)
-			delete [] temp_matrix[i];
-		delete [] temp_matrix;
+    ModelMarkov::decomposeRateMatrix();
     return;
-    }
 }
 
 void ModelPoMo::set_heterozygosity_boundaries() {
@@ -1047,7 +1093,6 @@ void ModelPoMo::set_heterozygosity_boundaries() {
 
 // TODO DS: The parameter mixture is unused at the moment.
 void ModelPoMo::computeTransMatrix(double time, double *trans_matrix, int mixture) {
-#ifdef USE_EIGEN3
   MatrixExpTechnique technique = phylo_tree->params->matrix_exp_technique;
   if (technique == MET_SCALING_SQUARING || !is_reversible) {
     // Do not change the object rate_matrix, but only trans_matrix.
@@ -1098,8 +1143,4 @@ void ModelPoMo::computeTransMatrix(double time, double *trans_matrix, int mixtur
   }
 
   else ModelMarkov::computeTransMatrix(time, trans_matrix);
-
-#else
-  ModelMarkov::computeTransMatrix(time, trans_matrix);
-#endif
 }
