@@ -535,12 +535,210 @@ model MTINV=
 2.751362222454670 0.072153227138698 0.065420927831618 0.088077392769029 1.555396905840990 0.065221940911213 0.231186486525368 0.396428077428706 0.008043434782625 8.542589373962890 1.075872855650690 0.056252073499162 1.509852171058890 0.534676724129225 0.162975872809625 0.377288666084473 1.885338569864270 0.128598566560553 0.080346083861554
 0.031742312696925 0.010900704360282 0.061579224631690 0.016149206459683 0.013570105428042 0.014644105857642 0.022311208924484 0.047847519139008 0.011641804656722 0.094322337728935 0.149407059762824 0.044438717775487 0.077262530905012 0.102287040914816 0.026290210516084 0.105939042375617 0.042869117147647 0.020701008280403 0.046556718622687 0.059540023816010;
 
+model Q.PFAM=
+0.531344742 
+0.266631781 0.610524242 
+0.479415354 0.145193836 4.395589145 
+2.490407258 0.797726764 0.617331366 0.086320436 
+1.058818226 2.850794598 1.685541958 0.623180282 0.163023963 
+1.178483844 0.358512949 0.572153867 4.775857514 0.004224284 4.045465925 
+1.897932882 0.427923043 1.417171473 0.993358642 0.723368327 0.349584077 0.412573692 
+0.453464468 2.765967911 4.395995003 0.944775779 1.220289602 4.992256584 0.444851397 0.432227777 
+0.176454495 0.103023046 0.192557924 0.012280201 0.599859067 0.090487083 0.045755066 0.025135568 0.108027826 
+0.419433650 0.307712278 0.070917051 0.019106538 0.827369996 0.609556427 0.066812844 0.070095729 0.420152907 4.316039810 
+0.501174376 5.070603955 2.126974783 0.311739636 0.042113153 3.211891588 1.628511729 0.323774881 0.779002069 0.188076678 0.128912693 
+1.175077280 0.565654138 0.405987508 0.044371788 1.330027314 1.704580053 0.217689890 0.196370346 0.488138895 5.052397990 6.674964742 0.790881216 
+0.266730243 0.050284344 0.098902029 0.023281590 1.570975979 0.044860498 0.016021778 0.116848629 0.754840320 1.333037626 2.539936322 0.022355802 1.944915128 
+1.371519672 0.396818483 0.230865860 0.538351193 0.117103191 0.764761023 0.532587532 0.323334201 0.635697033 0.101194285 0.285369684 0.446883087 0.146079999 0.106999973 
+3.745215188 0.711941869 3.559567653 1.188991720 3.084090802 1.178336151 0.587105765 1.645856748 1.047872072 0.078854093 0.182164122 0.756113129 0.449949835 0.319772739 1.499944856 
+2.134156546 0.664457704 1.923647217 0.498902533 1.382576296 1.172710577 0.733559201 0.232722239 0.648977470 1.067438502 0.303042511 1.169845557 2.176841262 0.185666747 0.667935113 5.538833003 
+0.198118657 0.503943335 0.048383536 0.030638664 0.964172901 0.185024339 0.044029570 0.199917400 0.592439554 0.143667666 0.614180565 0.040725071 0.765641182 2.174974075 0.133590865 0.217347672 0.125958817 
+0.208747809 0.263834003 0.570488409 0.134714779 1.863419357 0.264729772 0.100447410 0.074554161 5.545635324 0.271724216 0.338670344 0.138599247 0.651180870 7.474120415 0.108442089 0.374198514 0.267599595 2.604411280 
+2.688525915 0.201107356 0.119873351 0.052396485 2.371412865 0.282178057 0.297627071 0.134209258 0.229732340 11.786948184 2.030164484 0.222793132 2.397008325 0.758096789 0.362295352 0.127446562 2.284500453 0.201953893 0.337688120 
+0.085788000 0.057731000 0.042028000 0.056462000 0.010447000 0.039548000 0.067799000 0.064861000 0.021040000 0.055398000 0.100413000 0.059401000 0.019898000 0.042789000 0.039579000 0.069262000 0.055498000 0.014430000 0.033233000 0.064396000;
+
+model Q.PFAM_GB=
+0.365164838 
+0.267954861 0.668182618 
+0.412395647 0.109946883 5.109280517 
+2.729878231 0.826085083 0.726938192 0.087850737 
+0.877084208 2.867777221 1.690914729 0.549211875 0.175083471 
+1.030653110 0.296644302 0.531939174 5.408576458 0.009322812 4.263791122 
+2.118558237 0.406823565 1.377653412 0.873060903 0.782691675 0.293835651 0.404384209 
+0.350857800 2.639511515 4.957438759 0.957717365 1.116928692 5.496513866 0.357768891 0.337006729 
+0.117950785 0.097118281 0.189788694 0.007265312 0.427089565 0.083014733 0.034316792 0.014707735 0.086519266 
+0.265550676 0.256367291 0.059881044 0.010401807 0.642230229 0.589163556 0.053257602 0.037254822 0.365721292 3.923176859 
+0.457693311 5.846934542 2.118089709 0.248389767 0.027767460 3.282211527 1.530154625 0.267310479 0.632651172 0.148725361 0.106747583 
+0.983716229 0.449899179 0.360441883 0.019663070 1.089667442 1.579564776 0.178533992 0.144384262 0.390780590 5.076767492 6.772375029 0.740162467 
+0.184682627 0.034838076 0.079279076 0.013765656 1.373741900 0.033977644 0.013172775 0.066574829 0.649373699 0.993172189 2.231416771 0.015878425 1.671492831 
+1.208554025 0.298720644 0.164723116 0.340542665 0.126175761 0.713343366 0.393866840 0.185899793 0.585878841 0.068750403 0.259147052 0.350619140 0.101213184 0.074341327 
+4.632633836 0.692974365 4.084102769 1.010171060 3.782357424 1.053657420 0.483380661 1.759872389 0.936824568 0.067637494 0.172313759 0.617037785 0.391313179 0.312203677 1.419107393 
+2.317571767 0.564138173 2.051675662 0.400824407 1.655895774 1.036694976 0.582434600 0.163002252 0.527116986 1.125002936 0.262729464 1.039042636 2.278959095 0.137832417 0.601398020 6.382983692 
+0.107582893 0.435536872 0.047263468 0.026999980 0.817007211 0.206727272 0.030420354 0.141119644 0.501645686 0.086205240 0.414640307 0.024983931 0.528052784 1.764940773 0.070162265 0.201322252 0.086102239 
+0.157118656 0.205907035 0.544230162 0.119435501 1.653471491 0.208926712 0.080278936 0.047930460 5.372406910 0.169889476 0.258512719 0.100866499 0.460416547 7.845710592 0.075597432 0.317057267 0.177135947 2.300580433 
+2.411548529 0.146939179 0.105041792 0.041670032 2.240651656 0.235573727 0.256746635 0.094430216 0.142765142 12.429218156 1.730591322 0.181062532 2.353311726 0.601728375 0.266981380 0.093044732 2.271589526 0.130636191 0.212433758 
+0.087660000 0.058154000 0.037239000 0.048117000 0.013233000 0.038080000 0.063213000 0.059035000 0.021871000 0.061155000 0.111580000 0.056999000 0.022763000 0.046732000 0.035355000 0.065285000 0.052818000 0.015550000 0.035618000 0.069541000;
+
+model Q.LG=
+0.424057540 
+0.271250376 0.764825201 
+0.401980425 0.140794893 5.062142812 
+2.327453440 0.519625401 0.507639701 0.071616388 
+1.072849735 3.123615724 1.879768813 0.664578644 0.104402693 
+1.069798507 0.402468665 0.595330390 5.601033240 0.000110712 4.776313857 
+1.793212924 0.339928462 1.257892965 0.741295542 0.488516386 0.272681884 0.332128789 
+0.392890955 2.558126695 4.639398318 0.987385467 0.677157762 5.468173089 0.498257950 0.296507145 
+0.138014910 0.132223556 0.192515195 0.013803595 0.328676783 0.094404276 0.045491635 0.009271038 0.121398119 
+0.367358512 0.278306063 0.065286972 0.014185552 0.536563257 0.592378803 0.071724886 0.041522603 0.366698667 3.930811583 
+0.563978915 6.845986165 2.226004549 0.353825055 0.002829803 3.797608224 2.027714606 0.282308308 0.801983530 0.169487062 0.141910872 
+1.128143826 0.489462975 0.408708404 0.023806440 0.904437688 1.839460723 0.196671330 0.125388186 0.497496251 4.359960777 6.201341249 0.706300529 
+0.236258684 0.055016203 0.092864269 0.016998748 0.957089079 0.037738461 0.018574182 0.075225605 0.703446369 1.021152682 2.226104190 0.019952549 1.726515479 
+1.158823278 0.344518589 0.176377788 0.403179438 0.080797749 0.708201457 0.456455246 0.180072253 0.538060086 0.083244488 0.235273346 0.427986716 0.111914514 0.092858734 
+4.667976453 0.835065436 3.955582326 1.241154390 2.628965883 1.346684133 0.670392140 1.526697708 1.067966220 0.054379608 0.183116838 0.828615345 0.354736216 0.335899646 1.341323449 
+2.159899413 0.617964388 2.022264246 0.477448489 1.129405944 1.248400502 0.659780378 0.124572237 0.649139830 1.017998492 0.303426657 1.225974643 2.088967227 0.169151604 0.605969830 6.645417766 
+0.151862790 0.477052140 0.039141184 0.025562335 0.529678067 0.221132079 0.066497056 0.187213189 0.555296865 0.099536892 0.479069315 0.048841785 0.606743708 1.854514258 0.076842003 0.202206939 0.123868108 
+0.192903795 0.274767702 0.526873728 0.124040819 1.020373268 0.276139426 0.112342970 0.044355728 5.074607897 0.213437583 0.276570096 0.143396629 0.480201901 6.641252774 0.081147599 0.358925565 0.235545698 2.373266140 
+2.511548974 0.180665670 0.088670492 0.042435978 1.853035143 0.252220059 0.270634816 0.078641075 0.148865811 11.081231885 1.642242475 0.209350089 2.027034834 0.628362476 0.310510022 0.091930966 2.286257438 0.164177306 0.267739693 
+0.080009000 0.052947000 0.041171000 0.050146000 0.015018000 0.035929000 0.061392000 0.064793000 0.021709000 0.063895000 0.106292000 0.057047000 0.023440000 0.047712000 0.039604000 0.062980000 0.052863000 0.014987000 0.037434000 0.070634000;
+
+model Q.BIRD=
+0.086772353 
+0.041489234 0.145522693 
+0.684872641 0.038182359 4.588509426 
+0.214514127 2.310001798 0.094392928 0.085866464 
+0.104071310 4.326376802 0.119230449 0.051178971 0.037945162 
+0.768984102 0.089892499 0.080461407 5.114292481 0.028175939 1.988992665 
+2.215762795 1.937855330 0.295994975 2.041821920 1.073717504 0.105321639 1.420133178 
+0.080617856 6.927918567 3.653639043 1.071049041 2.585629399 6.123585849 0.058085432 0.108840477 
+0.046084289 0.203642191 0.465074580 0.025094905 0.142117407 0.031239309 0.016280777 0.029795942 0.047732049 
+0.116389136 0.336043365 0.021519291 0.026444908 0.269064319 0.527627637 0.038528060 0.047328310 0.607702969 1.935765980 
+0.093630870 5.781174762 2.258646191 0.026415889 0.011551984 1.637753572 1.761938656 0.087005765 0.004280896 0.148647886 0.044628305 
+0.235654912 0.732190858 0.048983640 0.016856105 0.129334019 0.168730118 0.077835842 0.072799197 0.035426840 7.398376755 3.334268838 0.708348805 
+0.092004559 0.061734224 0.047373731 0.043468817 1.575630749 0.045946765 0.043087788 0.049575370 0.141447452 0.962852023 2.950112395 0.040069839 0.134340383 
+2.022526978 0.407828168 0.021912937 0.057096272 0.124231767 1.446258879 0.060414502 0.063511937 1.485608296 0.042135216 1.450911972 0.054326874 0.048270788 0.072731066 
+2.352357809 0.961653423 5.657997764 0.161901882 2.848589688 0.086837961 0.056173980 2.558621780 0.248112786 0.287491221 0.653587912 0.105167714 0.055804307 0.936346554 2.818055661 
+9.189210510 0.711833578 1.856292472 0.041327738 0.117750493 0.075637987 0.075603922 0.085458940 0.064922501 3.670916550 0.086159175 0.827391264 6.908142924 0.086710531 1.546051817 3.320463486 
+0.067881151 2.240359335 0.029075520 0.024018688 2.572562133 0.667514386 0.057387721 0.561105174 0.032498470 0.045679288 0.687156290 0.024744157 0.188378332 0.454841057 0.054773510 0.269153898 0.048484013 
+0.062428150 0.069988164 0.632160724 0.452258389 5.266555093 0.049244809 0.032418984 0.035199737 6.764718412 0.083433936 0.094733583 0.036104364 0.043823350 3.894213263 0.056354401 0.461507798 0.048169854 0.308629484 
+5.126996382 0.063299974 0.037743293 0.240637305 0.199254052 0.052218599 0.307081397 0.848692284 0.058399592 13.397205976 2.035426441 0.054281453 7.643787012 0.780383422 0.081010240 0.082620150 0.391832532 0.082144494 0.059610809 
+0.066363000 0.054021000 0.037784000 0.047511000 0.022651000 0.048841000 0.071571000 0.058368000 0.025403000 0.045108000 0.100181000 0.061361000 0.021069000 0.038230000 0.053861000 0.089298000 0.053536000 0.012313000 0.027173000 0.065359000;
+
+model Q.INSECT=
+0.245103884 
+0.396680459 0.602596590 
+0.388851211 0.169418318 4.002207051 
+1.687379028 0.888477494 0.707887462 0.137794758 
+0.813368876 2.004043618 1.229351544 0.369955633 0.230607800 
+0.797312266 0.224720163 0.413998478 7.046040747 0.061906941 2.610915158 
+2.835188240 0.639959824 1.963743484 1.231717882 1.043646688 0.393592403 0.553182381 
+0.362258381 2.615490291 4.573260980 0.797011018 0.896230099 5.675152800 0.330930746 0.378124450 
+0.228513969 0.141228769 0.208490043 0.053173950 0.460428950 0.116150694 0.064283924 0.068021312 0.140874297 
+0.286953900 0.240779967 0.098667680 0.058795467 0.493466139 0.643789822 0.079309054 0.089730309 0.500992010 3.381560080 
+0.325994754 5.655671758 1.541274942 0.211396072 0.082097991 2.054032646 0.761883191 0.341199781 0.396451596 0.144888450 0.103298167 
+0.854941063 0.469236454 0.344916944 0.082042638 0.564344515 1.049351234 0.169182004 0.214023208 0.337275045 4.607657510 5.779760276 0.501772274 
+0.202912169 0.083503104 0.104497835 0.062211382 1.265119124 0.061813267 0.037371750 0.130615907 0.693502085 0.880367224 2.142804923 0.038837539 1.068089269 
+2.098306482 0.448091631 0.295630473 0.393309252 0.214758763 1.226945820 0.411528885 0.517165713 0.865174971 0.164903860 0.371130819 0.335358981 0.264037782 0.131445137 
+4.447741802 0.609758408 3.798086697 0.731000508 3.467889377 0.737657764 0.379154866 2.320367575 0.697254375 0.107585891 0.228053049 0.417871055 0.387566384 0.273240209 1.842353422 
+3.173624632 0.474607818 2.267368199 0.310254294 1.125638230 0.924249331 0.410818877 0.401697351 0.490764490 1.310627756 0.287148699 0.827466845 2.368908910 0.164073566 1.031968282 5.706447436 
+0.113618297 0.421211392 0.052343656 0.053998922 1.063048958 0.118187043 0.058461221 0.275756639 0.303820579 0.088741408 0.404916338 0.067974475 0.484518123 1.412153565 0.131745320 0.257205952 0.121214423 
+0.159463305 0.209585392 0.485673084 0.153134973 1.990179836 0.174718362 0.083149696 0.137582926 6.378949466 0.188267059 0.280539346 0.067140987 0.264484864 10.165067155 0.155778076 0.294006077 0.191009158 1.616501717 
+2.466363084 0.145305726 0.149535968 0.109375281 1.295115095 0.258250207 0.287452076 0.247675666 0.177412560 10.861831869 1.518647299 0.149072919 2.157390423 0.496751123 0.367029379 0.180581516 1.639361417 0.103038434 0.189789265 
+0.063003000 0.049585000 0.047550000 0.048622000 0.015291000 0.044058000 0.072012000 0.037810000 0.022358000 0.066563000 0.107325000 0.080621000 0.023976000 0.041578000 0.028532000 0.081767000 0.055167000 0.009698000 0.032219000 0.072265000;
+
+model Q.MAMMAL=
+0.164520503 
+0.133786660 0.301753825 
+0.667759687 0.102671327 4.669240542 
+0.375561674 1.878554130 0.218506732 0.132842206 
+0.172392722 3.619009961 0.250359878 0.124852096 0.095864103 
+0.781197529 0.208357490 0.202345038 5.379197337 0.056958677 1.746841825 
+2.076338500 1.797395963 0.679494500 1.991708503 1.179185026 0.168866582 1.453533780 
+0.161623558 4.710380602 3.940278094 0.956160014 1.796411770 5.600293701 0.164776154 0.227985715 
+0.203510704 0.217922574 0.520603634 0.071028795 0.233803448 0.064010956 0.054619397 0.086763961 0.094641478 
+0.178925750 0.403058144 0.048270492 0.042419644 0.297464704 0.583097899 0.070620812 0.086464596 0.627289885 2.383373587 
+0.150019883 5.862144662 2.011938106 0.109850770 0.049580542 1.705745478 1.499396112 0.184898463 0.108290367 0.192452773 0.074328424 
+0.426954296 0.737495653 0.124928924 0.058185276 0.192839562 0.273324822 0.142904022 0.172462927 0.119658259 7.422424301 4.121859707 0.760101625 
+0.156046906 0.098468307 0.074017139 0.075986849 1.649041128 0.061484905 0.059915230 0.098183513 0.275397853 1.183422487 2.957377967 0.060573566 0.338235196 
+1.899027463 0.525891518 0.068009985 0.123677157 0.200110404 1.590125986 0.137838674 0.136748511 1.445166153 0.107833373 1.381061149 0.129943936 0.137065096 0.146012906 
+2.359299936 0.843914331 5.427387027 0.263292147 2.507254736 0.189228647 0.139146074 2.427076788 0.376188219 0.330866332 0.498124825 0.193144225 0.150312624 0.827642694 2.786459548 
+7.524532664 0.539886133 1.955763604 0.132705770 0.233037817 0.165733855 0.155868082 0.198123473 0.145627442 3.528654954 0.153102275 0.889109054 4.911310946 0.119728464 1.529988220 3.075670543 
+0.101039395 2.131494369 0.058369398 0.043905868 2.144674342 0.521419600 0.088844939 0.579178348 0.134177423 0.088684660 0.683790243 0.064089109 0.276245610 0.603795093 0.100007515 0.264443242 0.079630880 
+0.091694198 0.139818982 0.655149301 0.420781565 5.685285320 0.126113134 0.070592995 0.083304052 7.287086510 0.172110578 0.176288430 0.080506454 0.165573576 5.877878503 0.112513257 0.458878972 0.102291161 0.645002709 
+4.300445029 0.107177686 0.082093349 0.228482448 0.278600785 0.094238794 0.325642029 0.868890049 0.081513165 12.950251994 1.879650916 0.097391390 7.158497660 0.733119107 0.145626117 0.152049202 0.573328824 0.113423271 0.125003102 
+0.067997000 0.055503000 0.036288000 0.046867000 0.021435000 0.050281000 0.068935000 0.055323000 0.026410000 0.041953000 0.101191000 0.060037000 0.019662000 0.036237000 0.055146000 0.096864000 0.057136000 0.011785000 0.024730000 0.066223000;
+
+model Q.PLANT=
+0.061995451 
+0.071787018 0.324146307 
+0.482723250 0.017012888 5.640569094 
+0.523802485 2.773840824 0.412259505 0.072474815 
+0.266699470 3.319795598 0.641219533 0.101391710 0.105145707 
+0.848857861 0.027378890 0.203417806 6.983296655 0.004325163 3.676487445 
+2.099517664 0.923332845 0.989386407 1.214523767 1.225782488 0.189953993 0.893983216 
+0.086540313 4.386107333 5.633764157 1.133417157 3.025390235 8.135368256 0.117555479 0.127760160 
+0.047711951 0.148339563 0.470979475 0.007582115 0.267133294 0.022845535 0.010786746 0.011450563 0.074259162 
+0.132461376 0.242868565 0.008688476 0.007966889 0.387974066 0.713657288 0.029290769 0.013401682 0.666044528 3.178490964 
+0.135231094 6.796751125 3.253593795 0.018324171 0.000109001 2.561068546 1.166641775 0.150166421 0.040648681 0.143807298 0.022265650 
+0.487603228 0.622545469 0.164436842 0.009922368 0.133016192 0.383963916 0.121953673 0.072782198 0.060192574 6.137883228 5.109951861 0.997263914 
+0.101998846 0.018497482 0.033489490 0.011945428 2.787245776 0.000109001 0.003428084 0.016479873 0.304517393 1.260751123 3.067538770 0.004231422 0.367280212 
+1.679096158 0.423800539 0.020342871 0.080963818 0.083580934 1.394642593 0.104298769 0.064165663 1.150907470 0.036382379 0.559761940 0.059246444 0.049078824 0.049489758 
+3.724953683 0.724887668 4.517199656 0.281847349 5.612663729 0.304709235 0.112097795 1.591990129 0.447466853 0.164991657 0.549645550 0.209386705 0.171531722 0.574024731 2.359864552 
+4.753201153 0.655493224 2.669910479 0.105335369 0.389742063 0.300884387 0.195263436 0.123875362 0.116237656 2.552275429 0.089626134 1.172764365 4.055778486 0.062695238 1.066677979 5.735630021 
+0.016624844 0.808384672 0.005560145 0.000347713 2.878143952 0.176461840 0.013106289 0.196264065 0.066883060 0.000109001 0.582218341 0.002231252 0.229612944 0.597613653 0.009791567 0.216845648 0.000109001 
+0.033775073 0.054821001 0.753418700 0.255631501 4.340476213 0.036965535 0.020444242 0.012699715 7.284279143 0.053844351 0.084699285 0.016433002 0.066804579 8.185198097 0.032534641 0.405822992 0.051593479 0.524555683 
+3.226243245 0.041421498 0.061468976 0.147004300 0.560195764 0.088095759 0.303691165 0.297902118 0.066305354 10.911840366 1.918251057 0.050145945 2.585318015 0.627525728 0.114608090 0.058704709 0.972725592 0.026833885 0.133140453 
+0.074923000 0.050500000 0.038734000 0.053195000 0.011300000 0.037499000 0.068513000 0.059627000 0.021204000 0.058991000 0.102504000 0.067306000 0.022371000 0.043798000 0.037039000 0.084451000 0.047850000 0.012322000 0.030777000 0.077097000;
+
+model Q.YEAST=
+0.289760345 
+0.342709634 0.718300668 
+0.367886518 0.072562600 4.199522650 
+3.691718604 0.710404342 0.490823952 0.052871678 
+1.043523521 2.008049414 1.553982959 0.449223552 0.070963821 
+1.093168815 0.139582694 0.508273036 5.624270628 0.003118838 3.707570018 
+2.882498207 0.540565704 1.987856240 1.230079486 1.182218128 0.398459557 0.542851891 
+0.417376590 2.637569081 4.921959964 0.793976563 0.971187101 5.448912766 0.394563504 0.492571299 
+0.182603367 0.132608825 0.173586122 0.017141968 0.595831116 0.083860488 0.036837325 0.027627162 0.116377119 
+0.334353188 0.205566131 0.068829530 0.021633405 0.796201770 0.465713591 0.062744857 0.043967523 0.378253079 3.366451402 
+0.483774690 6.326126715 1.772882594 0.193037560 0.025984034 2.595024049 0.980236499 0.351568331 0.667015573 0.108153720 0.090350953 
+0.980143366 0.618539982 0.354791389 0.052464777 0.984440777 1.122348190 0.146156313 0.190256883 0.392801743 4.449605122 5.986085778 0.578286681 
+0.224857913 0.047417869 0.073768891 0.020291850 1.625802715 0.031817027 0.015954525 0.094986525 0.713520963 0.952803363 2.116139937 0.012660509 1.303449237 
+1.902789461 0.327908181 0.288401051 0.418241394 0.119283083 0.994116603 0.453784839 0.361756388 0.717288404 0.097506652 0.200252021 0.383751236 0.146206205 0.091990755 
+4.877234074 0.595474107 3.112134717 0.924744908 3.584581583 1.239592330 0.512666905 1.863057260 0.955917766 0.058056067 0.241269232 0.551359124 0.486660696 0.253426381 1.572179322 
+2.789256813 0.577913042 2.172349980 0.379983795 1.746675706 1.004182700 0.576198956 0.295694231 0.636427012 1.138771705 0.291850288 0.980917255 1.905694316 0.158541858 0.735801862 5.167267165 
+0.082544434 0.297203203 0.023885222 0.009802221 1.098149199 0.061054054 0.021273070 0.132048920 0.320030703 0.103698871 0.411524750 0.030961092 0.442192031 1.474552928 0.057108109 0.136000409 0.072755518 
+0.176505390 0.179523334 0.380460546 0.103068007 1.987376163 0.160373469 0.057915259 0.060539607 6.419218432 0.171959625 0.282755433 0.046896769 0.361827346 7.660749919 0.085482550 0.275364684 0.182111094 1.812622556 
+2.936397672 0.163865947 0.135944973 0.080351380 2.828941339 0.271020707 0.294499028 0.133659895 0.255089465 11.291813609 1.513322757 0.167243117 2.085093473 0.620211936 0.348307577 0.145275987 2.188995240 0.093529663 0.240182683 
+0.059954000 0.042032000 0.052518000 0.054641000 0.008189000 0.040467000 0.070691000 0.039935000 0.018393000 0.069555000 0.109563000 0.081967000 0.018694000 0.046979000 0.031382000 0.091102000 0.055887000 0.010241000 0.033496000 0.064313000;
+
+model FLAVI=
+0.077462
+0.078037 0.000020
+0.550515 0.089476 8.801355
+0.114675 0.572845 0.000020 0.000020
+0.090490 3.856678 0.093133 0.183601 0.000020
+0.560685 0.020614 0.000020 7.603314 0.000020 1.058066
+0.478097 1.281339 0.147801 1.359685 0.221287 0.000020 2.331269
+0.168719 2.704348 3.326034 0.543235 0.240631 8.958548 0.025386 0.032838
+0.000020 0.151250 0.538307 0.000020 0.000020 0.000020 0.000020 0.000020 0.037032
+0.002576 0.114188 0.000020 0.000020 0.085459 0.789751 0.023842 0.037784 0.375616 1.571343
+0.000020 19.413475 2.764154 0.000020 0.000020 1.545589 2.992960 0.027884 0.000020 0.253247 0.038560
+0.000020 0.151495 0.000020 0.000020 0.000020 0.000020 0.031428 0.009423 0.000020 8.538171 2.895172 0.340189
+0.000020 0.000020 0.000020 0.000020 0.773082 0.000020 0.000020 0.000020 0.092713 1.368786 6.296781 0.000020 0.000020
+1.256282 0.292376 0.000020 0.000020 0.000020 1.571014 0.000020 0.029602 0.723179 0.000020 1.182805 0.097941 0.000020 0.035125
+2.104417 0.947496 9.494063 0.127754 1.516267 0.140959 0.000020 1.735582 0.079928 0.211717 1.008880 0.131204 0.073511 1.821906 4.417267
+6.315368 0.199572 1.329820 0.000020 0.000020 0.057328 0.000020 0.044927 0.144097 6.767791 0.033401 0.567475 2.228245 0.000020 1.543179 3.290965
+0.000020 0.791312 0.000020 0.000020 2.305902 0.000020 0.000020 0.323982 0.206837 0.000020 0.176249 0.000020 0.173206 0.148283 0.074193 0.154086 0.000020
+0.043583 0.000020 1.247952 0.771837 4.085694 0.000020 0.029684 0.000020 16.468674 0.000020 0.000020 0.071741 0.228965 5.907642 0.000020 0.431256 0.000020 0.000020
+8.030561 0.000020 0.000020 0.439809 0.149716 0.000020 0.346045 0.583259 0.067473 16.913225 1.074192 0.006629 3.855848 1.176807 0.036750 0.052613 0.134211 0.000020 0.234116
+0.077500 0.053813 0.033950 0.034973 0.014056 0.030139 0.054825 0.086284 0.018210 0.063272 0.103857 0.059646 0.040389 0.033630 0.036649 0.060915 0.076327 0.030152 0.020069 0.071343;
+
 end;
 )";
 
 
 ModelProtein::ModelProtein(const char *model_name, string model_params, StateFreqType freq, string freq_params, PhyloTree *tree, ModelsBlock* models_block)
- : ModelMarkov(tree)
+ : ModelMarkov(tree, true, false)
 {
     this->models_block = models_block;
 	init(model_name, model_params, freq, freq_params);
@@ -592,23 +790,66 @@ void ModelProtein::init(const char *model_name, string model_params, StateFreqTy
 
 	} else if (!model_params.empty()) {
         readParametersString(model_params);
+        rescaleRates(rates, getNumRateEntries());
         num_params = 0;
     } else if (name_upper == "GTR20") {
-        outWarning("GTR20 model will estimate 189 substitution rates that might be overfitting!");
-        outWarning("Please only use GTR20 with very large data and always test for model fit!");
+        if (!Params::getInstance().link_model) {
+            outWarning("GTR20 model will estimate 189 substitution rates that might be overfitting!");
+            outWarning("Please only use GTR20 with very large data and always test for model fit!");
+        }
         if (freq == FREQ_UNKNOWN)
             freq = FREQ_EMPIRICAL;
-        // initialize rate matrix with LG
-        nxs_model = models_block->findModel("LG");
-        ASSERT(nxs_model);
-        readParametersString(nxs_model->description);
-        rescaleRates(rates, getNumRateEntries());
-
+        if (Params::getInstance().model_name_init) {
+            nxs_model = models_block->findModel(Params::getInstance().model_name_init);
+            if (nxs_model) {
+                readParametersString(nxs_model->description, false);
+            } else {
+                // initialize with custom model file
+                readParameters(Params::getInstance().model_name_init, false);
+            }
+            rescaleRates(rates, getNumRateEntries());
+            if (!isReversible())
+                outError("Cannot initialize from non-reversible model");
+        } else {
+            // initialize rate matrix with LG
+            nxs_model = models_block->findModel("LG");
+            ASSERT(nxs_model);
+            readParametersString(nxs_model->description, false);
+            rescaleRates(rates, getNumRateEntries());
+        }
         // 2018-05-08 bug fix: GTR20 rates are not optimized
         num_params = getNumRateEntries()-1;
+    } else if (name_upper == "NONREV") {
+        if (!Params::getInstance().link_model) {
+            outWarning("NONREV model will estimate 379 substitution rates that might be overfitting!");
+            outWarning("Please only use NONREV with very large data and always test for model fit!");
+        }
+        if (freq == FREQ_UNKNOWN)
+            freq = FREQ_ESTIMATE;
+        if (Params::getInstance().model_name_init) {
+            nxs_model = models_block->findModel(Params::getInstance().model_name_init);
+            if (nxs_model) {
+                readParametersString(nxs_model->description, false);
+            } else {
+                // initialize with custom model file
+                readParameters(Params::getInstance().model_name_init, false);
+            }
+            rescaleRates(rates, getNumRateEntries());
+            if (isReversible())
+                setReversible(false);
+        } else {
+            // initialize rate matrix with LG
+            nxs_model = models_block->findModel("LG");
+            ASSERT(nxs_model);
+            readParametersString(nxs_model->description, false);
+            rescaleRates(rates, getNumRateEntries());
+            setReversible(false);
+        }
+        num_params = getNumRateEntries()-1;
 	} else {
 		// if name does not match, read the user-defined model
 		readParameters(model_name);
+        rescaleRates(rates, getNumRateEntries());
         num_params = 0;
 	}
 	if (freq_params != "") {
@@ -627,7 +868,7 @@ void ModelProtein::startCheckpoint() {
 
 
 void ModelProtein::saveCheckpoint() {
-    if (num_params > 0) {
+    if (num_params > 0 && !fixed_parameters) {
         startCheckpoint();
         CKP_ARRAY_SAVE(getNumRateEntries(), rates);
         endCheckpoint();
@@ -638,7 +879,7 @@ void ModelProtein::saveCheckpoint() {
 void ModelProtein::restoreCheckpoint() {
     ModelMarkov::restoreCheckpoint();
 
-    if (num_params > 0) {
+    if (num_params > 0 && !fixed_parameters) {
         startCheckpoint();
         CKP_ARRAY_RESTORE(getNumRateEntries(), rates);
         endCheckpoint();
@@ -694,3 +935,19 @@ void ModelProtein::readRates(istream &in) throw(const char*, string) {
 }
 
 
+string ModelProtein::getNameParams() {
+    ostringstream retname;
+    retname << name;
+    retname << freqTypeString(freq_type, phylo_tree->aln->seq_type, true);
+    
+    if (fixed_parameters)
+        return retname.str();
+
+    if (freq_type == FREQ_ESTIMATE) {
+        retname << "{" << state_freq[0];
+        for (int i = 1; i < num_states; i++)
+            retname << "," << state_freq[i];
+        retname << "}";
+    }
+    return retname.str();
+}
