@@ -71,7 +71,7 @@ public:
 	/**
 	 * @return model name with parameters in form of e.g. GTR{a,b,c,d,e,f}
 	 */
-	virtual string getNameParams() { return name; }
+    virtual string getNameParams();
 
 private:
     ModelsBlock *models_block;
