@@ -31,32 +31,37 @@ RateInvar::RateInvar(double p_invar_sites, PhyloTree *tree)
 	} else
 		p_invar = MIN_PINVAR;
 	fix_p_invar = false;
-    optimize_p_invar = true;
+//    optimize_p_invar = true;
 	phylo_tree = tree;
 	name = "+I";
 	full_name = "Invar";
 	if (p_invar_sites >= 0) {
 		p_invar = p_invar_sites;
-		fix_p_invar = true;
+		// true unless -optfromgiven cmd line option
+		fix_p_invar = !(Params::getInstance().optimize_from_given_params);
 	}
 }
 
-void RateInvar::saveCheckpoint() {
+void RateInvar::startCheckpoint() {
     checkpoint->startStruct("RateInvar");
+}
+
+void RateInvar::saveCheckpoint() {
+    startCheckpoint();
     CKP_SAVE(p_invar);
 //    CKP_SAVE(fix_p_invar);
 //    CKP_SAVE(optimize_p_invar);
-    checkpoint->endStruct();
+    endCheckpoint();
     RateHeterogeneity::saveCheckpoint();
 }
 
 void RateInvar::restoreCheckpoint() {
     RateHeterogeneity::restoreCheckpoint();
-    checkpoint->startStruct("RateInvar");
+    startCheckpoint();
     CKP_RESTORE(p_invar);
 //    CKP_RESTORE(fix_p_invar);
 //    CKP_RESTORE(optimize_p_invar);
-    checkpoint->endStruct();
+    endCheckpoint();
 }
 
 string RateInvar::getNameParams() {
@@ -90,7 +95,8 @@ void RateInvar::setBounds(double *lower_bound, double *upper_bound, bool *bound_
 double RateInvar::optimizeParameters(double gradient_epsilon) {
     if (phylo_tree->aln->frac_const_sites == 0.0)
         return -computeFunction(0.0);
-	if (fix_p_invar || !optimize_p_invar)
+//	if (fix_p_invar || !optimize_p_invar)
+    if (fix_p_invar)
 		return -computeFunction(p_invar);
 	if (verbose_mode >= VB_MAX)
 		cout << "Optimizing proportion of invariable sites..." << endl;
