@@ -2681,12 +2681,6 @@ void NxsCharactersBlock::Reset()
 	missing				= '?';
 	gap					= '\0';
 	matchchar			= '\0';
-	matrix				= NULL;
-	charPos				= NULL;
-	taxonPos			= NULL;
-	activeTaxon			= NULL;
-	activeChar			= NULL;
-	symbols				= NULL;
 
 	ResetSymbols();
 
@@ -2757,7 +2751,7 @@ void NxsCharactersBlock::ResetSymbols()
 			break;
 
 		case NxsCharactersBlock::protein:
-			strcpy(symbols, "ACDEFGHIKLMNPQRSTVWY*");
+			strcpy(symbols, "ACDEFGHIKLMNPQRSTVWY*XU");
 			break;
 
 		default:
@@ -2786,6 +2780,7 @@ void NxsCharactersBlock::ResetSymbols()
 		{
 		equates[ NxsString("B") ] = NxsString("{DN}");
 		equates[ NxsString("Z") ] = NxsString("{EQ}");
+        equates[ NxsString("J") ] = NxsString("{IL}");
 		}
 	}
 
