@@ -142,8 +142,13 @@ void MTree::copyTree(MTree *tree) {
 }
 
 void MTree::copyTree(MTree *tree, string &taxa_set) {
-    if (tree->leafNum != taxa_set.length())
-    	outError("#leaves and taxa_set do not match!");
+    rooted = tree->rooted;
+    if (rooted) {
+        ASSERT(tree->rooted);
+        taxa_set.push_back(1);
+    }
+//    if (tree->leafNum != taxa_set.length())
+//        outError("#leaves and taxa_set do not match!");
     leafNum = nodeNum = branchNum = 0;
     for (string::iterator it = taxa_set.begin(); it != taxa_set.end(); it++)
         nodeNum += (*it);
@@ -151,6 +156,8 @@ void MTree::copyTree(MTree *tree, string &taxa_set) {
     if (root) freeNode();
     root = NULL;
     root = copyTree(tree, taxa_set, new_len);
+    if (rooted)
+        ASSERT(root->name == ROOT_NAME);
 }
 
 Node* MTree::copyTree(MTree *tree, string &taxa_set, double &len, Node *node, Node *dad) {
@@ -381,6 +388,31 @@ void MTree::printTree(const char *ofile, int brtype)
     }
 }
 
+void MTree::printNexus(string ofile, int brtype, string nexus_comment)
+{
+    try {
+        ofstream out;
+        out.exceptions(ios::failbit | ios::badbit);
+        if (brtype & WT_APPEND)
+            out.open(ofile, ios_base::out | ios_base::app);
+        else
+            out.open(ofile);
+        out << "#NEXUS" << endl;
+        if (!nexus_comment.empty())
+            out << "[ " << nexus_comment << " ]" << endl;
+        out << "begin trees;" << endl;
+        out << "  tree tree_1 = ";
+        printTree(out, brtype | WT_BR_ATTR);
+        out << endl;
+        out << "end;" << endl;
+        out.close();
+        if (verbose_mode >= VB_DEBUG)
+            cout << "Tree was printed to " << ofile << endl;
+    } catch (ios::failure) {
+        outError(ERR_WRITE_OUTPUT, ofile);
+    }
+}
+
 void MTree::printTree(ostream &out, int brtype) {
     if (root->isLeaf()) {
         if (root->neighbors[0]->node->isLeaf()) {
@@ -428,18 +460,33 @@ struct IntStringCmp
 typedef set<IntString*, IntStringCmp> IntStringSet;
 
 void MTree::printBranchLength(ostream &out, int brtype, bool print_slash, Neighbor *length_nei) {
+    if (length_nei->length == -1.0)
+        return; // NA branch length
     int prec = 10;
 	double length = length_nei->length;
     if (brtype & WT_BR_SCALE) length *= len_scale;
     if (brtype & WT_BR_LEN_SHORT) prec = 6;
     if (brtype & WT_BR_LEN_ROUNDING) length = round(length);
     out.precision(prec);
+    if ((brtype & WT_BR_ATTR) && !length_nei->attributes.empty()) {
+        // print branch attributes
+        out << "[&";
+        bool first = true;
+        for (auto attr : length_nei->attributes) {
+            if (!first)
+                out << ",";
+            out << attr.first << "=\"" << attr.second << '"';
+            first = false;
+        }
+        out << "]";
+    }
+    
     if (brtype & WT_BR_LEN) {
         if (brtype & WT_BR_LEN_FIXED_WIDTH)
             out << ":" << fixed << length;
         else
             out << ":" << length;
-    } else if (brtype & WT_BR_CLADE) {
+    } else if (brtype & WT_BR_CLADE && length_nei->node->name != ROOT_NAME) {
     	if (print_slash)
     		out << "/";
         out << length;
@@ -516,10 +563,10 @@ int MTree::printTree(ostream &out, int brtype, Node *node, Node *dad)
                 delete (*iss);
         }
         out << ")";
-        if (!node->name.empty())
-            out << node->name;
-        else if (brtype & WT_INT_NODE)
+        if (brtype & WT_INT_NODE)
             out << node->id;
+        else if (!node->name.empty() && (brtype & WT_BR_ATTR) == 0)
+            out << node->name;
         if (dad != NULL || length_nei) {
         	printBranchLength(out, brtype, !node->name.empty(), length_nei);
         }
@@ -678,7 +725,7 @@ void MTree::readTree(istream &in, bool &is_rooted)
         Node *node;
         parseFile(in, ch, node, branch_len);
         // 2018-01-05: assuming rooted tree if root node has two children
-        if (is_rooted || !branch_len.empty() || node->degree() == 2) {
+        if (is_rooted || (!branch_len.empty() && branch_len[0] != 0.0) || node->degree() == 2) {
             if (branch_len.empty())
                 branch_len.push_back(-1.0);
             if (branch_len[0] == -1.0) branch_len[0] = 0.0;
@@ -689,6 +736,7 @@ void MTree::readTree(istream &in, bool &is_rooted)
             root->addNeighbor(node, branch_len);
             node->addNeighbor(root, branch_len);
             leafNum++;
+            rooted = true;
         } else { // assign root to one of the neighbor of node, if any
             FOR_NEIGHBOR_IT(node, NULL, it)
             if ((*it)->node->isLeaf()) {
@@ -1011,6 +1059,18 @@ void MTree::getInternalNodes(NodeVector &nodes, Node *node, Node *dad) {
     }
 }
 
+void MTree::getMultifurcatingNodes(NodeVector &nodes, Node *node, Node *dad) {
+    if (!node) node = root;
+    //for (NeighborVec::iterator it = node->neighbors.begin(); it != node->neighbors.end(); it++)
+    //if ((*it)->node != dad)    {
+    FOR_NEIGHBOR_IT(node, dad, it)
+    if (!(*it)->node->isLeaf()) {
+        if ((*it)->node->degree() > 3)
+            nodes.push_back((*it)->node);
+        getMultifurcatingNodes(nodes, (*it)->node, node);
+    }
+}
+
 void MTree::generateNNIBraches(NodeVector &nodes1, NodeVector &nodes2, SplitGraph* excludeSplits, Node *node, Node *dad) {
     if (!node) node = root;
     //for (NeighborVec::iterator it = node->neighbors.begin(); it != node->neighbors.end(); it++)
@@ -1082,7 +1142,34 @@ bool MTree::isABranch(Node* node1, Node* node2) {
     return (node1->findNeighbor(node2) != NULL && node2->findNeighbor(node1) != NULL);
 }
 
-void MTree::getBranches(NodeVector &nodes, NodeVector &nodes2, Node *node, Node *dad) {
+void MTree::getBranches(NodeVector &nodes, NodeVector &nodes2, Node *node, Node *dad, bool post_traversal) {
+    if (!node) node = root;
+    //for (NeighborVec::iterator it = node->neighbors.begin(); it != node->neighbors.end(); it++)
+    //if ((*it)->node != dad)   {
+    FOR_NEIGHBOR_IT(node, dad, it) {
+        if (!post_traversal) {
+            if (node->id < (*it)->node->id) {
+                nodes.push_back(node);
+                nodes2.push_back((*it)->node);
+            } else {
+                nodes.push_back((*it)->node);
+                nodes2.push_back(node);
+            }
+        }
+        getBranches(nodes, nodes2, (*it)->node, node, post_traversal);
+        if (post_traversal) {
+            if (node->id < (*it)->node->id) {
+                nodes.push_back(node);
+                nodes2.push_back((*it)->node);
+            } else {
+                nodes.push_back((*it)->node);
+                nodes2.push_back(node);
+            }
+        }
+    }
+}
+
+void MTree::getBranches(int max_dist, NodeVector &nodes, NodeVector &nodes2, Node *node, Node *dad) {
     if (!node) node = root;
     //for (NeighborVec::iterator it = node->neighbors.begin(); it != node->neighbors.end(); it++)
     //if ((*it)->node != dad)   {
@@ -1094,7 +1181,27 @@ void MTree::getBranches(NodeVector &nodes, NodeVector &nodes2, Node *node, Node
             nodes.push_back((*it)->node);
             nodes2.push_back(node);
         }
-        getBranches(nodes, nodes2, (*it)->node, node);
+        if (max_dist > 1)
+            getBranches(max_dist-1, nodes, nodes2, (*it)->node, node);
+    }
+}
+
+void MTree::getBranches(BranchVector& branches, Node *node, Node *dad, bool post_traversal) {
+    if (!node) node = root;
+    FOR_NEIGHBOR_IT(node, dad, it) {
+        if (!post_traversal) {
+            Branch branch;
+            branch.first = node;
+            branch.second = (*it)->node;
+            branches.push_back(branch);
+        }
+        getBranches(branches, (*it)->node, node, post_traversal);
+        if (post_traversal) {
+            Branch branch;
+            branch.first = node;
+            branch.second = (*it)->node;
+            branches.push_back(branch);
+        }
     }
 }
 
@@ -1111,6 +1218,25 @@ void MTree::getInnerBranches(Branches& branches, Node *node, Node *dad) {
     }
 }
 
+void MTree::getInnerBranches(BranchVector& branches, Node *node, Node *dad, bool post_traversal) {
+    if (!node) node = root;
+    FOR_NEIGHBOR_IT(node, dad, it) {
+        if (!node->isLeaf() && !(*it)->node->isLeaf() && !post_traversal) {
+            Branch branch;
+            branch.first = node;
+            branch.second = (*it)->node;
+            branches.push_back(branch);
+        }
+        getInnerBranches(branches, (*it)->node, node, post_traversal);
+        if (!node->isLeaf() && !(*it)->node->isLeaf() && post_traversal) {
+            Branch branch;
+            branch.first = node;
+            branch.second = (*it)->node;
+            branches.push_back(branch);
+        }
+    }
+}
+
 void MTree::getBranchLengths(vector<DoubleVector> &len, Node *node, Node *dad) {
     if (!node) {
         node = root;
@@ -1164,6 +1290,16 @@ void MTree::getTaxaName(vector<string> &taxname, Node *node, Node *dad) {
     }
 }
 
+void MTree::getNodeName(set<string> &nodename, Node *node, Node *dad) {
+    if (!node) node = root;
+    if (!node->name.empty())
+        nodename.insert(node->name);
+    //for (NeighborVec::iterator it = node->neighbors.begin(); it != node->neighbors.end(); it++)
+    //if ((*it)->node != dad)    {
+    FOR_NEIGHBOR_IT(node, dad, it) {
+        getNodeName(nodename, (*it)->node, node);
+    }
+}
 
 void MTree::getUnorderedTaxaName(vector<string> &taxname, Node *node, Node *dad) {
     if (!node) node = root;
@@ -1350,6 +1486,13 @@ void MTree::convertToTree(SplitGraph &sg) {
     root = newnode;
     nodeNum++;
     cladetaxa.clear();
+    string root_name = ROOT_NAME;
+    newnode = findLeafName(root_name);
+    if (newnode) {
+        rooted = true;
+        root = newnode;
+    }
+    
 }
 
 Node *MTree::findNodeName(string &name, Node *node, Node *dad) {
@@ -1547,6 +1690,42 @@ string MTree::reportInputInfo() {
     return str;
 }
 
+void MTree::convertToUnrooted() {
+    ASSERT(rooted && root);
+    ASSERT(root->isLeaf() && root->id == leafNum-1);
+    Node *node = root->neighbors[0]->node;
+    Node *taxon = findFirstTaxon();
+    
+    rooted = false;
+    leafNum--;
+    
+    // delete root node
+    if (node->degree() == 3) {
+        // delete and join adjacent branches
+        Node *node1 = NULL, *node2 = NULL;
+        double len = 0.0;
+        FOR_NEIGHBOR_IT(node, root, it) {
+            if (!node1) node1 = (*it)->node; else node2 = (*it)->node;
+            len += (*it)->length;
+        }
+        node1->updateNeighbor(node, node2, len);
+        node2->updateNeighbor(node, node1, len);
+        delete node;
+    } else {
+        // only delete root node
+        auto it = node->findNeighborIt(root);
+        delete *it;
+        node->neighbors.erase(it);
+        
+    }
+    
+    delete root;
+    // set a temporary taxon so that tree traversal works
+    root = taxon;
+    
+    initializeTree();
+    //    computeBranchDirection();
+}
 
 typedef map<int, Neighbor*> IntNeighborMap;
 
@@ -1940,46 +2119,71 @@ void MTree::getTaxa(Split &taxa, Node *node, Node *dad) {
 }
 
 
-void MTree::extractQuadSubtrees(vector<Split*> &subtrees, Node *node, Node *dad) {
+void MTree::extractQuadSubtrees(vector<Split*> &subtrees, BranchVector &branches, Node *node, Node *dad) {
 	if (!node) node = root;
 	FOR_NEIGHBOR_IT(node, dad, it) {
-		extractQuadSubtrees(subtrees, (*it)->node, node);
+		extractQuadSubtrees(subtrees, branches, (*it)->node, node);
 		if ((*it)->node->isLeaf()) continue;
 		// internal branch
 		ASSERT(node->degree() == 3 && (*it)->node->degree() == 3);
 		int cnt = 0;
 		Node *child = (*it)->node;
+        string treestrings[4];
+        int nodeid = 0;
 		FOR_NEIGHBOR_DECLARE(child, node, it2) {
 			Split *sp = new Split(leafNum);
 			getTaxa(*sp, (*it2)->node, child);
 			subtrees.push_back(sp);
 			cnt += sp->countTaxa();
+            if (Params::getInstance().print_df1_trees) {
+                ostringstream ostr;
+                printTree(ostr, WT_BR_LEN, (*it2)->node, child);
+                treestrings[nodeid] = ostr.str();
+                treestrings[nodeid] = treestrings[nodeid].substr(0, treestrings[nodeid].length()-1);
+                nodeid++;
+            }
 		}
 		FOR_NEIGHBOR(node, child, it2) {
 			Split *sp = new Split(leafNum);
 			getTaxa(*sp, (*it2)->node, node);
 			subtrees.push_back(sp);
 			cnt += sp->countTaxa();
+            if (Params::getInstance().print_df1_trees) {
+                ostringstream ostr;
+                printTree(ostr, WT_BR_LEN, (*it2)->node, node);
+                treestrings[nodeid] = ostr.str();
+                treestrings[nodeid] = treestrings[nodeid].substr(0, treestrings[nodeid].length()-1);
+                nodeid++;
+            }
 		}
 		ASSERT(cnt == leafNum);
+        branches.push_back({node, child});
+        if (Params::getInstance().print_df1_trees) {
+            ASSERT(nodeid == 4);
+            // output NNI-1 tree
+            string treeDF1 = "(" + treestrings[0] + "," + treestrings[2] + ",(" + treestrings[1] + "," + treestrings[3] + "));";
+            string treeDF2 = "(" + treestrings[0] + "," + treestrings[3] + ",(" + treestrings[1] + "," + treestrings[2] + "));";
+            PUT_ATTR(child->findNeighbor(node), treeDF1);
+            PUT_ATTR(child->findNeighbor(node), treeDF2);
+        }
 	}
 }
 
-
-void MTree::assignBranchSupport(const char *trees_file) {
+/*
+void MTree::assignBranchSupport(const char *trees_file, map<int,BranchSupportInfo> &branch_supports) {
 	cout << "Reading input trees file " << trees_file << endl;
 	try {
 		ifstream in;
         in.exceptions(ios::failbit | ios::badbit);
         in.open(trees_file);
-        assignBranchSupport(in);
+        assignBranchSupport(in, branch_supports);
 		in.close();
 	} catch (ios::failure) {
 		outError(ERR_READ_INPUT, trees_file);
 	}
 }
 
-void MTree::assignBranchSupport(istream &in) {
+void MTree::assignBranchSupport(istream &in, map<int,BranchSupportInfo> &branch_supports) {
 	SplitGraph mysg;
 	NodeVector mynodes;
 	convertSplits(mysg, &mynodes, root->neighbors[0]->node);
@@ -2083,16 +2287,31 @@ void MTree::assignBranchSupport(istream &in) {
 	for (int i = 0; i < mysg.size(); i++)
 	if (!mynodes[i]->isLeaf())
 	{
+        BranchSupportInfo brsup;
+        brsup.id = mynodes[i]->id;
+        brsup.name = mynodes[i]->name;
+        brsup.geneCF = mysg[i]->getWeight()/decisive_counts[i];
+        brsup.geneN = decisive_counts[i];
+        branch_supports[brsup.id] = brsup;
+        
 		stringstream tmp;
-		if (!mynodes[i]->name.empty())
-			tmp << "/";
 		if (mysg[i]->getWeight() == 0.0)
 			tmp << "0";
 		else
 			tmp << round((mysg[i]->getWeight()/decisive_counts[i])*1000)/10;
 		if (verbose_mode >= VB_MED)
 			tmp << "%" << decisive_counts[i];
-		if (!mynodes[i]->isLeaf()) mynodes[i]->name.append(tmp.str());
+
+        if (Params::getInstance().newick_extended_format) {
+            if (mynodes[i]->name.empty() || mynodes[i]->name.back() != ']')
+                mynodes[i]->name += "[&CF=" + tmp.str() + "]";
+            else
+                mynodes[i]->name = mynodes[i]->name.substr(0, mynodes[i]->name.length()-1) + ",!CF=" + tmp.str() + "]";
+        } else {
+            if (!mynodes[i]->name.empty())
+                mynodes[i]->name.append("/");
+            mynodes[i]->name.append(tmp.str());
+        }
 		if (verbose_mode >= VB_MED) {
 			cout << mynodes[i]->name << " " << occurence_trees[i] << endl;
 		}
@@ -2100,27 +2319,37 @@ void MTree::assignBranchSupport(istream &in) {
 	for (vector<Split*>::reverse_iterator it = subtrees.rbegin(); it != subtrees.rend(); it++)
 		delete (*it);
 }
+*/
 
-void MTree::computeRFDist(const char *trees_file, IntVector &dist) {
+void MTree::computeRFDist(const char *trees_file, DoubleVector &dist, int assign_sup) {
 	cout << "Reading input trees file " << trees_file << endl;
 	try {
 		ifstream in;
         in.exceptions(ios::failbit | ios::badbit);
         in.open(trees_file);
-        computeRFDist(in, dist);
+        computeRFDist(in, dist, assign_sup);
 		in.close();
 	} catch (ios::failure) {
 		outError(ERR_READ_INPUT, trees_file);
 	}
 }
 
-void MTree::computeRFDist(istream &in, IntVector &dist) {
+void MTree::computeRFDist(istream &in, DoubleVector &dist, int assign_sup, bool one_tree) {
 	SplitGraph mysg;
-	convertSplits(mysg, NULL, root->neighbors[0]->node);
+    NodeVector nodes;
+	convertSplits(mysg, &nodes, root->neighbors[0]->node);
+    StringIntMap name_index;
+    int ntrees, taxid;
+    for (taxid = 0; taxid < mysg.getNTaxa(); taxid++)
+        name_index[mysg.getTaxa()->GetTaxonLabel(taxid)] = taxid;
+    NodeVector::iterator nit;
+    if (assign_sup)
+        for (nit = nodes.begin(); nit != nodes.end(); nit++)
+            (*nit)->height = 0.0;
+    
 	SplitGraph::iterator sit;
 	for (sit = mysg.begin(); sit != mysg.end(); sit++)
 		(*sit)->setWeight(0.0);
-	int ntrees, taxid;
 	for (ntrees = 1; !in.eof(); ntrees++) {
 		MTree tree;
 		bool is_rooted = false;
@@ -2134,9 +2363,9 @@ void MTree::computeRFDist(istream &in, IntVector &dist) {
 		// create the map from taxa between 2 trees
 		Split taxa_mask(leafNum);
 		for (StrVector::iterator it = taxname.begin(); it != taxname.end(); it++) {
-			taxid = mysg.findLeafName(*it);
-			if (taxid < 0)
-				outError("Taxon not found in full tree: ", *it);
+            if (name_index.find(*it) == name_index.end())
+                outError("Taxon not found in full tree: ", *it);
+			taxid = name_index[*it];
 			taxa_mask.addTaxon(taxid);
 		}
 		// make the taxa ordering right before converting to split system
@@ -2180,12 +2409,18 @@ void MTree::computeRFDist(istream &in, IntVector &dist) {
 							cout << " " << taxname[taxid];
 					cout << endl;
 				}
+                if (assign_sup && subsp->trivial() < 0)
+                    nodes[sit-mysg.begin()]->height++;
 			}
 			delete subsp;
 		}
 
 		//cout << "common_splits = " << common_splits << endl;
-        int rf_val = branchNum-leafNum + tree.branchNum-tree.leafNum - 2*common_splits;
+        double max_dist = branchNum-leafNum + tree.branchNum-tree.leafNum;
+        double rf_val = max_dist - 2*common_splits;
+        if (Params::getInstance().normalize_tree_dist) {
+            rf_val = rf_val / max_dist;
+        }
 		dist.push_back(rf_val);
 		char ch;
 		in.exceptions(ios::goodbit);
@@ -2193,8 +2428,15 @@ void MTree::computeRFDist(istream &in, IntVector &dist) {
 		if (in.eof()) break;
 		in.unget();
 		in.exceptions(ios::failbit | ios::badbit);
+        
+        if (one_tree)
+            break;
 
 	}
+    if (assign_sup)
+        for (nit = nodes.begin(); nit != nodes.end(); nit++)
+            if (!(*nit)->isLeaf())
+                (*nit)->name = convertIntToString((*nit)->height);
 
 //	cout << ntrees << " trees read" << endl;
 
@@ -2214,7 +2456,7 @@ void MTree::reportDisagreedTrees(vector<string> &taxname, MTreeSet &trees, Split
 }
 
 
-void MTree::createBootstrapSupport(vector<string> &taxname, MTreeSet &trees, SplitGraph &sg, SplitIntMap &hash_ss,
+void MTree::createBootstrapSupport(vector<string> &taxname, MTreeSet &trees, SplitIntMap &hash_ss,
     char *tag, Node *node, Node *dad) {
 	if (!node) node = root;	
 	FOR_NEIGHBOR_IT(node, dad, it) {
@@ -2242,7 +2484,7 @@ void MTree::createBootstrapSupport(vector<string> &taxname, MTreeSet &trees, Spl
 				  tmp << "/" << sp->getWeight();
                   
                 // assign tag
-                if (tag && (strcmp(tag, "ALL")==0 || (*it)->node->name == tag))
+                if (tag && (iEquals(tag, "ALL") || (*it)->node->name == tag))
                     tmp << sp->getName();                
 				(*it)->node->name.append(tmp.str());
 			} else {
@@ -2259,7 +2501,7 @@ void MTree::createBootstrapSupport(vector<string> &taxname, MTreeSet &trees, Spl
 				reportDisagreedTrees(taxname, trees, mysplit);
 			}
 		}
-		createBootstrapSupport(taxname, trees, sg, hash_ss, tag, (*it)->node, node);
+		createBootstrapSupport(taxname, trees, hash_ss, tag, (*it)->node, node);
 	}	
 }
 
