@@ -70,6 +70,29 @@ public:
         size = 0;
     }
 
+    /**
+     construct class with another Neighbor
+     @param nei another Neighbor
+     */
+    PhyloNeighbor(PhyloNeighbor *nei) : Neighbor(nei) {
+        partial_lh = NULL;
+        scale_num = NULL;
+        partial_lh_computed = 0;
+        lh_scale_factor = 0.0;
+        partial_pars = NULL;
+        direction = nei->direction;
+        size = nei->size;
+    }
+
+    
+    /**
+     allocate a new Neighbor by just copying from this one
+     @return pointer to newly created Neighbor
+     */
+    virtual Neighbor* newNeighbor() {
+        return (new PhyloNeighbor(this));
+    }
+
     /**
         tell that the partial likelihood vector is not computed
      */
