@@ -22,6 +22,7 @@
 #include "alignment/superalignmentpairwise.h"
 #include "main/phylotesting.h"
 #include "model/partitionmodel.h"
+#include "utils/MPIHelper.h"
 
 PhyloSuperTree::PhyloSuperTree()
  : IQTree()
@@ -31,7 +32,7 @@ PhyloSuperTree::PhyloSuperTree()
 	// Initialize the counter for evaluated NNIs on subtrees. FOR THIS CASE IT WON'T BE initialized.
 }
 
-PhyloSuperTree::PhyloSuperTree(SuperAlignment *alignment) :  IQTree(alignment) {
+PhyloSuperTree::PhyloSuperTree(SuperAlignment *alignment, bool new_iqtree) :  IQTree(alignment) {
     totalNNIs = evalNNIs = 0;
 
     rescale_codon_brlen = false;
@@ -50,7 +51,11 @@ PhyloSuperTree::PhyloSuperTree(SuperAlignment *alignment) :  IQTree(alignment) {
 
     StrVector model_names;
     for (it = alignment->partitions.begin(); it != alignment->partitions.end(); it++, part++) {
-        PhyloTree *tree = new PhyloTree((*it));
+        PhyloTree *tree;
+        if (new_iqtree)
+            tree = new IQTree(*it);
+        else
+            tree = new PhyloTree(*it);
         push_back(tree);
         PartitionInfo info;
         info.cur_ptnlh = NULL;
@@ -199,6 +204,12 @@ void PhyloSuperTree::setLikelihoodKernel(LikelihoodKernel lk) {
         (*it)->setLikelihoodKernel(lk);
 }
 
+void PhyloSuperTree::setParsimonyKernel(LikelihoodKernel lk) {
+    PhyloTree::setParsimonyKernel(lk);
+    for (iterator it = begin(); it != end(); it++)
+        (*it)->setParsimonyKernel(lk);
+}
+
 void PhyloSuperTree::changeLikelihoodKernel(LikelihoodKernel lk) {
 	PhyloTree::changeLikelihoodKernel(lk);
 }
@@ -209,6 +220,34 @@ void PhyloSuperTree::setNumThreads(int num_threads) {
         (*it)->setNumThreads((size() >= num_threads) ? 1 : num_threads);
 }
 
+void PhyloSuperTree::printResultTree(string suffix) {
+    if (MPIHelper::getInstance().isWorker()) {
+        return;
+    }
+    if (params->suppress_output_flags & OUT_TREEFILE)
+        return;
+    
+    IQTree::printResultTree(suffix);
+    
+    string tree_file_name = params->out_prefix;
+    tree_file_name += ".parttrees";
+    if (suffix.compare("") != 0) {
+        tree_file_name += "." + suffix;
+    }
+    try {
+        ofstream out;
+        out.exceptions(ios::failbit | ios::badbit);
+        out.open(tree_file_name);
+        for (iterator it = begin(); it != end(); it++)
+            (*it)->printTree(out, WT_BR_LEN | WT_BR_LEN_FIXED_WIDTH | WT_SORT_TAXA | WT_NEWLINE);
+        out.close();
+    } catch (ios::failure) {
+        outError(ERR_WRITE_OUTPUT, tree_file_name);
+    }
+    if (verbose_mode >= VB_MED)
+        cout << "Partition trees printed to " << tree_file_name << endl;
+}
+
 string PhyloSuperTree::getTreeString() {
 	stringstream tree_stream;
 	printTree(tree_stream, WT_TAXON_ID + WT_BR_LEN + WT_SORT_TAXA);
@@ -229,7 +268,7 @@ void PhyloSuperTree::readTreeString(const string &tree_string) {
 	setRootNode(params->root);
 	for (iterator it = begin(); it != end(); it++) {
 		(*it)->freeNode();
-		(*it)->readTree(str, rooted);
+		(*it)->readTree(str, (*it)->rooted);
         (*it)->assignLeafNames();
 //		(*it)->setAlignment((*it)->aln);
 	}
@@ -430,6 +469,32 @@ void PhyloSuperTree::linkTree(int part, NodeVector &part_taxa, SuperNode *node,
 	linkBranch(part, nei, dad_nei);
 }
 
+void PhyloSuperTree::syncRooting() {
+    if (empty() || !front()->root)
+        return;
+    // check if all trees are similarly rooted or unrooted
+    bool same_rooting = true;
+    iterator tree;
+    for (tree = begin(); tree != end(); tree++) {
+        if ((*tree)->rooted != front()->rooted) {
+            same_rooting = false;
+            break;
+        }
+    }
+    
+    if (same_rooting) {
+        if (!empty() && rooted != front()->rooted) {
+            if (rooted)
+                convertToUnrooted();
+            else
+                convertToRooted();
+        }
+    } else if (!rooted) {
+        // not same rooting between trees
+        convertToRooted();
+    }
+}
+
 void PhyloSuperTree::printMapInfo() {
 	NodeVector nodes1, nodes2;
 	getBranches(nodes1, nodes2);
@@ -466,6 +531,7 @@ void PhyloSuperTree::printMapInfo() {
 
 void PhyloSuperTree::mapTrees() {
 	ASSERT(root);
+    syncRooting();
 	int part = 0, i;
 	if (verbose_mode >= VB_DEBUG)
 		drawTree(cout,  WT_BR_SCALE | WT_INT_NODE | WT_TAXON_ID | WT_NEWLINE | WT_BR_ID);
@@ -482,7 +548,16 @@ void PhyloSuperTree::mapTrees() {
 		(*it)->getOrderedTaxa(my_taxa);
 		part_taxa.resize(leafNum, NULL);
 		for (i = 0; i < leafNum; i++) {
-			int id = ((SuperAlignment*)aln)->taxa_index[i][part];
+            int id;
+            if (i < aln->getNSeq())
+                id = ((SuperAlignment*)aln)->taxa_index[i][part];
+            else {
+                ASSERT(rooted);
+                if ((*it)->rooted)
+                    id = (*it)->leafNum-1;
+                else
+                    id = -1;
+            }
 			if (id >=0) part_taxa[i] = my_taxa[id];
 		}
 		if (verbose_mode >= VB_DEBUG) {
@@ -510,7 +585,13 @@ void PhyloSuperTree::linkTrees() {
 		part_taxa.resize(leafNum, NULL);
 		int i;
 		for (i = 0; i < leafNum; i++) {
-			int id = ((SuperAlignment*)aln)->taxa_index[i][part];
+            int id;
+            if (i < aln->getNSeq())
+                id = ((SuperAlignment*)aln)->taxa_index[i][part];
+            else if ((*it)->rooted)
+                id = (*it)->leafNum-1;
+            else
+                id = -1;
 			if (id >=0) part_taxa[i] = my_taxa[id];
 		}
 		linkTree(part, part_taxa);
@@ -589,6 +670,14 @@ void PhyloSuperTree::computePartitionOrder() {
         
     delete [] cost;
     delete [] id;
+    
+    if (verbose_mode >= VB_MED) {
+        cout << "Partitions ordered by computation costs:" << endl;
+        cout << "#nexus" << endl << "begin sets;" << endl;
+        for (i = 0; i < ntrees; i++)
+            cout << "  charset " << at(part_order[i])->aln->name << " = " << at(part_order[i])->aln->position_spec << ";" << endl;
+        cout << "end;" << endl;
+    }
 #else
     for (i = 0; i < ntrees; i++) {
         part_order[i] = i;
@@ -749,6 +838,12 @@ int PhyloSuperTree::getMaxPartNameLength() {
 }
 
 NNIMove PhyloSuperTree::getBestNNIForBran(PhyloNode *node1, PhyloNode *node2, NNIMove *nniMoves) {
+    if (((PhyloNeighbor*)node1->findNeighbor(node2))->direction == TOWARD_ROOT) {
+        // swap node1 and node2 if the direction is not right, only for nonreversible models
+        PhyloNode *tmp = node1;
+        node1 = node2;
+        node2 = tmp;
+    }
     NNIMove myMove;
     //myMove.newloglh = 0;
 	SuperNeighbor *nei1 = ((SuperNeighbor*)node1->findNeighbor(node2));
@@ -757,7 +852,9 @@ NNIMove PhyloSuperTree::getBestNNIForBran(PhyloNode *node1, PhyloNode *node2, NN
 	SuperNeighbor *node1_nei = NULL;
 	SuperNeighbor *node2_nei = NULL;
 	SuperNeighbor *node2_nei_other = NULL;
-	FOR_NEIGHBOR_DECLARE(node1, node2, node1_it) {
+	FOR_NEIGHBOR_DECLARE(node1, node2, node1_it)
+    if (((PhyloNeighbor*)*node1_it)->direction != TOWARD_ROOT)
+    {
 		node1_nei = (SuperNeighbor*)(*node1_it);
 		break;
 	}
@@ -988,7 +1085,7 @@ void PhyloSuperTree::doNNI(NNIMove &move, bool clearLH) {
 
 }
 
-void PhyloSuperTree::changeNNIBrans(NNIMove move) {
+void PhyloSuperTree::changeNNIBrans(NNIMove &move) {
 	SuperNeighbor *nei1 = (SuperNeighbor*)move.node1->findNeighbor(move.node2);
 	SuperNeighbor *nei2 = (SuperNeighbor*)move.node2->findNeighbor(move.node1);
 	iterator it;
@@ -1119,6 +1216,32 @@ uint64_t PhyloSuperTree::getMemoryRequired(size_t ncategory, bool full_mem) {
 	return mem_size;
 }
 
+// get memory requirement for ModelFinder
+uint64_t PhyloSuperTree::getMemoryRequiredThreaded(size_t ncategory, bool full_mem) {
+    // only get the largest k partitions (k=#threads)
+    int threads = (params->num_threads != 0) ? params->num_threads : params->num_threads_max;
+    threads = min(threads, countPhysicalCPUCores());
+    threads = min(threads, (int)size());
+    
+    // sort partition by computational cost for OpenMP effciency
+    uint64_t *part_mem = new uint64_t[size()];
+    int i;
+    for (i = 0; i < size(); i++) {
+        part_mem[i] = at(i)->getMemoryRequired(ncategory, full_mem);
+    }
+    
+    // sort partition memory in increasing order
+    quicksort<uint64_t, int>(part_mem, 0, size()-1);
+    
+    uint64_t mem = 0;
+    for (i = size()-threads; i < size(); i++)
+        mem += part_mem[i];
+    
+    delete [] part_mem;
+    
+    return mem;
+}
+
 int PhyloSuperTree::countEmptyBranches(PhyloNode *node, PhyloNode *dad) {
 	int count = 0;
     if (!node)
@@ -1299,11 +1422,11 @@ void PhyloSuperTree::writeSiteLh(ostream &out, SiteLoglType wsl, int partid) {
         (*it)->writeSiteLh(out, wsl, part);
 }
 
-void PhyloSuperTree::writeSiteRates(ostream &out, int partid) {
+void PhyloSuperTree::writeSiteRates(ostream &out, bool bayes, int partid) {
 
     int part = 1;
     for (iterator it = begin(); it != end(); it++, part++) {
-        (*it)->writeSiteRates(out, part);
+        (*it)->writeSiteRates(out, bayes, part);
     }
 }
 
@@ -1355,3 +1478,38 @@ void PhyloSuperTree::writeBranches(ostream &out) {
     }
 }
 
+void PhyloSuperTree::printBestPartitionParams(const char *filename) {
+    try {
+        ofstream out;
+        out.exceptions(ios::failbit | ios::badbit);
+        out.open(filename);
+        out << "#nexus" << endl
+        << "begin sets;" << endl;
+        int part;
+        SuperAlignment *saln = (SuperAlignment*)aln;
+        for (part = 0; part < size(); part++) {
+            string name = saln->partitions[part]->name;
+            replace(name.begin(), name.end(), '+', '_');
+            out << "  charset " << name << " = ";
+            if (!saln->partitions[part]->aln_file.empty()) out << saln->partitions[part]->aln_file << ": ";
+            if (saln->partitions[part]->seq_type == SEQ_CODON)
+                out << "CODON, ";
+            string pos = saln->partitions[part]->position_spec;
+            replace(pos.begin(), pos.end(), ',' , ' ');
+            out << pos << ";" << endl;
+        }
+        out << "  charpartition mymodels =" << endl;
+        for (part = 0; part < size(); part++) {
+            string name = saln->partitions[part]->name;
+            replace(name.begin(), name.end(), '+', '_');
+            if (part > 0) out << "," << endl;
+            out << "    " << at(part)->getModelNameParams() << ": " << name << "{" << at(part)->treeLength() << "}";
+        }
+        out << ";" << endl;
+        out << "end;" << endl;
+        out.close();
+        cout << "Partition information was printed to " << filename << endl;
+    } catch (ios::failure &) {
+        outError(ERR_WRITE_OUTPUT, filename);
+    }
+}
