@@ -104,7 +104,14 @@ void PhyloTreeMixlen::setMixlen(int mixlen) {
 	this->mixlen = mixlen;
 }
 
-void PhyloTreeMixlen::initializeModel(Params &params, string &model_name, ModelsBlock *models_block) {
+void PhyloTreeMixlen::readTreeString(const string &tree_string) {
+    IQTree::readTreeString(tree_string);
+    treeLengths(relative_treelen);
+    if (mixlen > 0 && relative_treelen[0] == 0.0)
+        relative_treelen.clear();
+}
+
+void PhyloTreeMixlen::initializeModel(Params &params, string model_name, ModelsBlock *models_block) {
     try {
         if (!getModelFactory()) {
             setModelFactory(new ModelFactoryMixlen(params, model_name, this, models_block));
@@ -618,7 +625,7 @@ void PhyloTreeMixlen::printBranchLength(ostream &out, int brtype, bool print_sla
                     out << fixed << length;
                 else
                     out << length;
-            } else if (brtype & WT_BR_CLADE) {
+            } else if (brtype & WT_BR_CLADE && length_nei->node->name != ROOT_NAME) {
                 out << length;
             }
         }
@@ -627,7 +634,7 @@ void PhyloTreeMixlen::printBranchLength(ostream &out, int brtype, bool print_sla
 
     if (brtype & WT_BR_LEN)
         out << ":";
-    else if ((brtype & WT_BR_CLADE) && print_slash)
+    else if ((brtype & WT_BR_CLADE) && print_slash && length_nei->node->name != ROOT_NAME)
         out << "/";
         
     double length = nei->length;
@@ -644,7 +651,7 @@ void PhyloTreeMixlen::printBranchLength(ostream &out, int brtype, bool print_sla
             out << fixed << length;
         else
             out << length;
-    } else if (brtype & WT_BR_CLADE) {
+    } else if (brtype & WT_BR_CLADE && length_nei->node->name != ROOT_NAME) {
         out << length;
     }
 }
@@ -784,8 +791,10 @@ void PhyloTreeMixlen::computeFuncDerv(double value, double &df, double &ddf) {
 	    	for (ptn = 0; ptn < nptn; ptn++) {
 				double *partial_lh_dad = dad_branch->partial_lh + ptn*block;
 				double *theta = theta_all + ptn*block;
+                
+                // TODO: check with vectorclass!
 				double *lh_tip = tip_partial_lh +
-						((int)((ptn < orig_nptn) ? (aln->at(ptn))[dad->id] :  model_factory->unobserved_ptns[ptn-orig_nptn]))*statemix;
+						((int)((ptn < orig_nptn) ? (aln->at(ptn))[dad->id] :  model_factory->unobserved_ptns[ptn-orig_nptn][dad->id]))*statemix;
 				for (m = 0; m < nmixture; m++) {
 					for (i = 0; i < statecat; i++) {
 						theta[m*statecat+i] = lh_tip[m*nstates + i%nstates] * partial_lh_dad[m*statecat+i];
