@@ -307,6 +307,7 @@ const int WT_BR_LEN_FIXED_WIDTH = 256;
 const int WT_BR_ID = 512;
 const int WT_BR_LEN_ROUNDING = 1024;
 const int WT_BR_LEN_SHORT = 2048; // store only 6 digits after the comma for branch lengths
+const int WT_BR_ATTR = 4096; // print branch attributes
 const int TRUE = 1;
 const int FALSE = 0;
 
@@ -399,18 +400,27 @@ enum StateFreqType {
     FREQ_DNA_2113, FREQ_DNA_2131, FREQ_DNA_2311, 
 };
 
-/**
-        alignment format type
+/*
+    outfile file format
+ FORMAT_NORMAL: usual file format used so far
+ FORMAT_CSV: csv file format
+ FORMAT_TSV: tab separated file format
  */
-
-enum AlnFormat {
-    ALN_PHYLIP, ALN_FASTA
+enum FileFormat {
+    FORMAT_NORMAL, FORMAT_CSV, FORMAT_TSV
 };
 
 enum ModelTestCriterion {
     MTC_AIC, MTC_AICC, MTC_BIC, MTC_ALL
 };
 
+/**
+ PartitionFinder merging algorithm
+ */
+enum PartitionMerge {
+    MERGE_NONE, MERGE_GREEDY, MERGE_RCLUSTER, MERGE_RCLUSTERF, MERGE_KMEANS
+};
+
 /**
         Stopping condition type
  */
@@ -427,7 +437,7 @@ enum LEAST_SQUARE_VAR {
 };
 
 enum START_TREE_TYPE {
-	STT_BIONJ, STT_PARSIMONY, STT_PLL_PARSIMONY, STT_RANDOM_TREE
+	STT_BIONJ, STT_PARSIMONY, STT_PLL_PARSIMONY, STT_RANDOM_TREE, STT_USER_TREE
 };
 
 const int MCAT_LOG = 1; // categorize by log(rate) for Meyer & von Haeseler model
@@ -481,14 +491,27 @@ enum MatrixExpTechnique {
     MET_LIE_MARKOV_DECOMPOSITION
 };
 
+/** ascertainment bias correction type */
+enum ASCType {
+    ASC_NONE, // no ASC
+    ASC_VARIANT, // Lewis's correction for variant sites
+    ASC_VARIANT_MISSING, // Holder's correction for variant sites with missing data
+    ASC_INFORMATIVE, // correction for parsimony-informative sites
+    ASC_INFORMATIVE_MISSING // Holder's correction for informative sites with missing data
+};
+
 enum AncestralSeqType {
     AST_NONE, AST_MARGINAL, AST_JOINT
 };
 
+enum SymTest {
+    SYMTEST_NONE, SYMTEST_BINOM, SYMTEST_MAXDIV
+};
 
 const int BRLEN_OPTIMIZE = 0; // optimize branch lengths
 const int BRLEN_FIX      = 1; // fix branch lengths
 const int BRLEN_SCALE    = 2; // scale branch lengths
+const int TOPO_UNLINKED  = 3; // unlinked/separate tree topologies between partitions
 
 const int OUT_LOG       = 1; // .log file written or not
 const int OUT_TREEFILE  = 2; // .treefile file written or not
@@ -629,6 +652,9 @@ public:
 	 */
 	int sprDist;
 
+    /** cost matrix file for Sankoff parsimony */
+    char *sankoff_cost_file;
+    
 	/**
 	 *  Number of NNI locally optimal trees generated from the set of parsimony trees
 	 *  Default = 20 (out of 100 parsimony trees)
@@ -657,6 +683,11 @@ public:
 	 *  logl epsilon for model parameter optimization
 	 */
 	double modelEps;
+    
+    /**
+     logl epsilon for ModelFinder
+     */
+    double modelfinder_eps;
 
 	/**
 	 *  New search heuristics (DEFAULT: ON)
@@ -789,6 +820,12 @@ public:
     /* type of starting tree */
     START_TREE_TYPE start_tree;
 
+    /** TRUE to infer fast ML tree for ModelFinder */
+    bool modelfinder_ml_tree;
+    
+    /** TRUE to perform final model optimization */
+    bool final_model_opt;
+    
     /** name of constraint tree file in NEWICK format */
     char *constraint_tree_file;
 
@@ -805,14 +842,54 @@ public:
     /** true if sequential phylip format is used, default: false (interleaved format) */
     bool phylip_sequential_format;
 
+    /**
+     SYMTEST_NONE to not perform test of symmetry of Jermiin et al. (default)
+     SYMTEST_MAXDIV to perform symmetry test on the pair with maximum divergence
+     SYMTEST_BINOM to perform binomial test of all pair p-values
+    */
+    SymTest symtest;
+    
+    /** TRUE to do symtest then exist */
+    bool symtest_only;
+    
+    /**
+     1 to remove bad loci by SymTest
+     2 to remove good loci by SymTest
+     */
+    int symtest_remove;
+    
+    /** true to keep zero which may result in many taxon pairs not testable (default: false) */
+    bool symtest_keep_zero;
+    
+    /**
+        which test used when removing loci
+        0 test of symmetry (default)
+        1 test of marginal symmetry
+        2 test of internal symmetry
+     */
+    int symtest_type;
+    
+    /** pvalue cutoff (default: 0.05) */
+    double symtest_pcutoff;
+
+    /** TRUE to print all pairwise statistics */
+    double symtest_stat;
+
+    /** Times to shuffle characters within columns of the alignment */
+    int symtest_shuffle;
+
     /**
             file containing multiple trees to evaluate at the end
      */
-    char *treeset_file;
+    string treeset_file;
 
     /** number of bootstrap replicates for tree topology test */
     int topotest_replicates;
 
+    /** TRUE to optimize model parameters for topology test,
+     FALSE (default) to only optimize branch lengths */
+    bool topotest_optimize_model;
+
     /** true to perform weighted SH and KH test */
     bool do_weighted_test;
 
@@ -833,21 +910,33 @@ public:
      */
     int partition_type;
 
+    /** PartitionFinder algorithm, default MERGE_NONE */
+    PartitionMerge partition_merge;
+    
     /** percentage for rcluster algorithm like PartitionFinder */
     double partfinder_rcluster; 
 
     /** absolute limit on #partition pairs for rcluster algorithm */
     size_t partfinder_rcluster_max;
 
-    /** rclusterf algorithm to top 50% pairs instead of only one best pair */
-    bool partfinder_rcluster_fast;
+    /** set of models used for model merging phase */
+    string merge_models;
 
+    /** set of rate models used for model merging phase */
+    string merge_rates;
+
+    /** use logarithm of rates for clustering algorithm */
+    bool partfinder_log_rate;
+    
     /** remove all-gap sequences in partition model to account for terrace default: TRUE */
     bool remove_empty_seq;
 
     /** use terrace aware data structure for partition models, default: TRUE */
     bool terrace_aware;
 
+    /** check if the tree lies on a terrace */
+    bool terrace_analysis;
+
     /**
             B, D, or P for Binary, DNA, or Protein sequences
      */
@@ -889,8 +978,18 @@ public:
     /**
             alignment output format
      */
-    AlnFormat aln_output_format;
+    InputType aln_output_format;
+    
+    /**
+        output file format
+     */
+    FileFormat output_format;
 
+    /**
+     tree in extended newick format with node label like [&label=""]
+     */
+    bool newick_extended_format;
+    
     /**
             TRUE to discard all gappy positions
      */
@@ -1090,6 +1189,20 @@ public:
      */
     bool is_rooted;
 
+    /**
+        maximum distance to move root
+     */
+    int root_move_dist;
+
+    /**
+     TRUE to find best root when optimizing model
+     */
+    bool root_find;
+
+    /**
+     TRUE to test all rooting positions at the end of the run
+     */
+    bool root_test;
 
     /**
             min branch length, used to create random tree/network
@@ -1146,6 +1259,25 @@ public:
     */
     char *support_tag;
 
+    /**
+        number of quartets for site concordance factor
+     */
+    int site_concordance;
+
+    /**
+     TRUE to print concordant sites per partition
+     */
+    bool site_concordance_partition;
+    
+    /** TRUE to print sCF for all sampled quartet */
+    bool print_cf_quartets;
+
+    /** TRUE to print trees associated with discordance factor 1 (NNI-1 tree) */
+    bool print_df1_trees;
+    
+    /** 1 to compute internode certainty */
+    int internode_certainty;
+    
     /**
             2nd alignment used in computing multinomialProb (Added by MA)
      */
@@ -1294,8 +1426,14 @@ public:
      */
     string model_name;
 
+    /** model name to initialize GTR20 or NONREV protein model */
+    char* model_name_init;
+
+    /** number of steps for linked model optimisation, default: 1 */
+    int model_opt_steps;
+    
     /** set of models for testing */
-    char *model_set;
+    string model_set;
 
     /** set of models to be added into default set */
     char *model_extra_set;
@@ -1307,11 +1445,17 @@ public:
     char *state_freq_set;
 
     /** set of rate heterogeneity model for testing */
-    char *ratehet_set;
+    string ratehet_set;
 
+    /** all models with score worse than the best score + this threshold will be ignored */
+    double score_diff_thres;
+    
     /** model defition file */
     char *model_def_file;
 
+    /** TRUE to perform ModelOMatic method of Whelan et al. 2015 */
+    bool modelomatic;
+    
     /** true to redo model testing even if .model file exists */
     bool model_test_again;
 
@@ -1346,6 +1490,9 @@ public:
     /** FALSE to set zero state frequency to 1e-4.
         Default: FALSE (version <= 1.5.5), TRUE (ver >= 1.5.6) */
     bool keep_zero_freq;
+    
+    /** minimal state frequency for optimisation, default=0.0001 */
+    double min_state_freq;
 
 
     /**
@@ -1456,6 +1603,15 @@ public:
     */
     char *bootstrap_spec;
 
+    /** 1 or 2 to perform transfer boostrap expectation (TBE) */
+    int transfer_bootstrap;
+    
+    /** subsampling some number of partitions / sites for analysis */
+    int subsampling;
+
+    /** random seed number for subsampling */
+    int subsampling_seed;
+
     /**
             1 if output all intermediate trees (initial trees, NNI-optimal trees and trees after each NNI step)
             2 if output all intermediate trees + 1-NNI-away trees
@@ -1485,6 +1641,16 @@ public:
      */
     int rf_dist_mode;
 
+    /**
+     true to compute distance between the same k-th tree in two sets
+     */
+    bool rf_same_pair;
+    
+    /**
+     true to normalize tree distances, false otherwise
+     */
+    bool normalize_tree_dist;
+    
     /**
             compute the site-specific rates by Meyer & von Haeseler method
      */
@@ -1580,8 +1746,12 @@ public:
     */
     SiteFreqType print_site_state_freq;
 
-    /** TRUE to print site-specific rates, default: FALSE */
-    bool print_site_rate;
+    /**
+     0 (default): do not print .rate file
+     1: print site-specific rates by empirical Bayes
+     2: site-specific rates by maximum-likelihood
+     */
+    int print_site_rate;
 
     /* 1: print site posterior probability for many trees during tree search */
     int print_trees_site_posterior;
@@ -1879,6 +2049,9 @@ public:
     
     /** maximum number of threads, default: #CPU scores  */
     int num_threads_max;
+    
+    /** true to parallel ModelFinder by models instead of sites */
+    bool openmp_by_model;
 
     /** either MTC_AIC, MTC_AICc, MTC_BIC */
     ModelTestCriterion model_test_criterion;
@@ -1912,6 +2085,12 @@ public:
 	/** TRUE to link alpha among Gamma model over partitions */
 	bool link_alpha;
 
+    /** TRUE to link substitution models over partitions */
+    bool link_model;
+
+    /** name of the joint model across partitions */
+    char* model_joint;
+    
 	/** true to count all distinct trees visited during tree search */
 	bool count_trees;
 
@@ -1928,6 +2107,9 @@ public:
 	 * 0: store all partial likelihood vectors
 	 * 1: only store 1 partial likelihood vector per node */
 	LhMemSave lh_mem_save;
+    
+    /** true to save buffer, default: false */
+    bool buffer_mem_save;
 
     /** maximum size of memory allowed to use */
     double max_mem_size;
@@ -1935,6 +2117,10 @@ public:
 	/* TRUE to print .splits file in star-dot format */
 	bool print_splits_file;
     
+    /* TRUE to print .splits.nex file in NEXUS format */
+    bool print_splits_nex_file;
+
+    
     /** TRUE (default) to ignore identical sequences and add them back at the end */
     bool ignore_identical_seqs;
 
@@ -1952,6 +2138,9 @@ public:
     /** true to compute sequence identity along tree */
     bool compute_seq_identity_along_tree;
     
+    /** true to compute sequence composition */
+    bool compute_seq_composition;
+    
     /** true to ignore checkpoint file */
     bool ignore_checkpoint;
     /** number of quartets for likelihood mapping */
@@ -1991,7 +2180,35 @@ public:
 	bool ufboot2corr; // to turn on the correction mode for UFBoot under model violations, enable by "-bb <nrep> -correct
 	bool u2c_nni5; // to use NNI5 during Refinement Step of UFBoot2-Corr
 
+    /** method for phylogenetic dating, currently only LSD is supported */
+    string dating_method;
 
+    /** extra commands passed to the dating method */
+    string dating_options;
+
+    /** date file that has several lines, each line with a taxon name and date in YYYY-MM-DD */
+    string date_file;
+    
+    /** tip date, a real number or YYYY-MM-DD */
+    string date_tip;
+    
+    /** root date, a real number or YYYY-MM-DD */
+    string date_root;
+    
+    /** false to remove outgroup from the dated tree, default: true */
+    bool date_with_outgroup;
+    
+    /** true to print internal date files for debugging purpose */
+    bool date_debug;
+    
+    /** number of replicates to compute date confidence interval */
+    int date_replicates;
+    
+    /** standard deviation of lognormal relaxed clock model for confidence interval estimate */
+    double clock_stddev;
+
+    /** z-score for detecting outlier nodes */
+    double date_outlier;
 };
 
 /**
@@ -2028,6 +2245,32 @@ struct PDRelatedMeasures {
 
 
 /*--------------------------------------------------------------*/
+    
+inline size_t get_safe_upper_limit(size_t cur_limit) {
+    if (Params::getInstance().SSE >= LK_AVX512)
+        // AVX-512
+        return ((cur_limit+7)/8)*8;
+    else
+        if (Params::getInstance().SSE >= LK_AVX)
+            // AVX
+            return ((cur_limit+3)/4)*4;
+        else
+            // SSE
+            return ((cur_limit+1)/2)*2;
+}
+
+inline size_t get_safe_upper_limit_float(size_t cur_limit) {
+    if (Params::getInstance().SSE >= LK_AVX512)
+        // AVX-512
+        return ((cur_limit+15)/16)*16;
+    else
+        if (Params::getInstance().SSE >= LK_AVX)
+            // AVX
+            return ((cur_limit+7)/8)*8;
+        else
+            // SSE
+            return ((cur_limit+3)/4)*4;
+}
 /*--------------------------------------------------------------*/
 
 /**
@@ -2154,6 +2397,12 @@ string convertInt64ToString(int64_t number);
 
 string convertDoubleToString(double number);
 
+/**
+ case-insensitive comparison between two strings
+ @return true if two strings are equal.
+ */
+bool iEquals(const string a, const string b);
+    
 /**
  *
  * @param SRC
@@ -2169,6 +2418,19 @@ bool copyFile(const char SRC[], const char DEST[]);
  */
 bool fileExists(string strFilename);
 
+/**
+    check that path is a directory
+ */
+int isDirectory(const char *path);
+
+/**
+    get all file names in a directory
+    @param path directory name
+    @param[out] filenames vector of file names
+    return 0 if FAIL, non-zero otherwise
+ */
+int getFilesInDir(const char *path, StrVector &filenames);
+
 /**
         convert string to int, with error checking
         @param str original string
@@ -2176,6 +2438,13 @@ bool fileExists(string strFilename);
  */
 int convert_int(const char *str);
 
+/**
+    convert string to int64, with error checking
+    @param str original string
+    @return the number
+ */
+int64_t convert_int64(const char *str);
+
 /**
         convert string to int, with error checking
         @param str original string
@@ -2255,7 +2524,7 @@ void convert_range(const char *str, int &lower, int &upper, int &step_size);
  */
 void convert_range(const char *str, double &lower, double &upper, double &step_size);
 
-void convert_string_vec(const char *str, StrVector &str_vec);
+void convert_string_vec(const char *str, StrVector &str_vec, char separator = ',');
 
 /**
     change unusual character in names into underscore (_)
